package ibspan.ecap.merchant.shopdecisions;

import java.io.IOException;
import java.util.Date;
import java.util.GregorianCalendar;

import java.util.Calendar;

import ibspan.ecap.merchant.shopdecisions.dataadministration.DemandRequest;
import ibspan.ecap.shared.util.TimeConverter;
import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.CyclicBehaviour;
import jade.lang.acl.ACLMessage;
import jade.util.Logger;

public class DemandInfoRequester extends CyclicBehaviour
{
	private Logger myLogger = Logger.getMyLogger(this.getClass().getName());
	
	private AID shopAgent;
	private int updateHour;
	private int updateMinute;
	private int updateSecond;
	private Date nextUpdate;
	private ProductProvider products;

	public DemandInfoRequester(Agent agent,
			AID shopAgent, int updateHour, int updateMinute, int updateSecond, ProductProvider provider)
	{
		super(agent);
		this.shopAgent = shopAgent;
		this.updateHour = updateHour;
		this.updateMinute = updateMinute;
		this.updateSecond = updateSecond;
		this.products = provider;
		
		setNextUpdateTime();
	}

	@Override
	public void action()
	{
		//myLogger.info("DemandInfoRequester running.");
		TimeConverter tc = TimeConverter.getInstance();
		Date now = tc.getDate();
		if(now.after(nextUpdate))
		{
			String nextProductId = products.getNextProductId();
			if (nextProductId != null)
			{
				myLogger.info("DemandInfoRequester sending demand request of product: "+ nextProductId);
				sendDemandRequest(nextProductId);
			}
			if(products.roundFinished())
			{
				myLogger.info("DemandInfoRequester sent all demand requests. Scheduling next update cycle.");
				setNextUpdateTime();	
				long timeTillNextUpdate = tc.convertSpanFromReal(nextUpdate.getTime() - now.getTime());
				myLogger.info("DemandInfoRequester blocking for :"+timeTillNextUpdate);
				block(timeTillNextUpdate);
			}
		}		
		else
		{
			long timeTillNextUpdate = tc.convertSpanFromReal(nextUpdate.getTime() - now.getTime());
			myLogger.info("DemandInfoRequester woken up to early. Now: "+now+"; next update: "+nextUpdate+". Blocking for :"+timeTillNextUpdate);
			block(timeTillNextUpdate);
		}
	}

	private void setNextUpdateTime()
	{
		TimeConverter tc = TimeConverter.getInstance();
		Date now = tc.getDate();
		
		Calendar cal = new GregorianCalendar();
		cal.setTime(now);
		
		cal.set(Calendar.HOUR_OF_DAY, updateHour);
		cal.set(Calendar.MINUTE, updateMinute);
		cal.set(Calendar.SECOND, updateSecond);
		
		if(cal.getTime().after(now))
		{
			//the current time is earlier than the scheduled update time this day
			nextUpdate = cal.getTime();
		}
		else
		{
			//the current time is later than the scheduled update time this day -> schedule the update for the following day
			cal.add(Calendar.DATE, 1);
			nextUpdate = cal.getTime();
		}
		myLogger.fine("Next update scheduled for "+nextUpdate);
	}

	private void sendDemandRequest(String productId)
	{
		TimeConverter tc = TimeConverter.getInstance();
		Date now = tc.getDate();
		
		Calendar cal = new GregorianCalendar();
		cal.setTime(now);
		
		cal.add(Calendar.DATE, -1);	//we're asking about yesterday
		
		
		Date yesterday = cal.getTime();
		
		DemandRequest req = new DemandRequest();
		req.setDate(yesterday);
		req.setProductId(productId); 
		
		ACLMessage demandReq = new ACLMessage(ACLMessage.REQUEST);
		demandReq.addReceiver(shopAgent);
		demandReq.setOntology("DemandRequest");				
		try
		{
			demandReq.setContentObject(req);
		} catch (IOException e)
		{				
			e.printStackTrace();
			throw new RuntimeException("Error setting demand data request content.",e);
		}            
		myAgent.send(demandReq);
	}

}
