package ibspan.ecap.merchant.shopdecisions;

import java.util.logging.Level;

import jade.core.Agent;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.CyclicBehaviour;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.util.Logger;

/**
 * A behaviour for receiving the message specified by the given template passing it to the 
 * DataStore of an appropriate handling behaviour created by the {@link #createHandlerBehaviour}
 * method and putting that behaviour into the agent's queue. The default implementation of the
 * {@link #createHandlerBehaviour} creates no behaviour, hence no handler is scheduled after 
 * receiving a message. To specify the behaviour to handle the received messages one should
 * subclass ReceivingWrapper class overriding {@link #createHandlerBehaviour}.
 *  
 * @author Micha Drozdowicz
 *
 */
public class ReceivingWrapper extends CyclicBehaviour {

	protected Logger myLogger = Logger.getMyLogger(this.getClass().getName());
	
	private MessageTemplate template;

	private Object dataStoreKey;

	public ReceivingWrapper(Agent myAgent, MessageTemplate template, Object dataStoreKey) {
		super(myAgent);

		this.template = template;
		this.dataStoreKey = dataStoreKey;
	}

	@Override
	public void action() {
		ACLMessage message = null;
		
		if(myLogger.isLoggable(Level.FINEST))
			myLogger.finest("Trying to receive message.Template: "+template.toString());
		message = myAgent.receive(template);
		if (message == null)
		{			
			block();
			return;			
		}
		
		Behaviour behaviour = createHandlerBehaviour();
		if(behaviour!=null)
		{
			behaviour.getDataStore().put(dataStoreKey, message);
			
			myLogger.finest("Received message. Adding child behaviour.");
			myAgent.addBehaviour(behaviour);
		}
	}
	
	/**
	 * Creates a behaviour object handling the received message. The default implementation 
	 * returns null, hence the method should be overriden in a subclass.  
	 *
	 * @return a behaviour to add to the agent's queue.
	 */
	protected Behaviour createHandlerBehaviour()
	{
		return null;
	}

}
