package ibspan.ecap.merchant.shopdecisions;

import java.util.Hashtable;

import ibspan.ecap.merchant.shopdecisions.dataadministration.DataAccessException;
import ibspan.ecap.merchant.shopdecisions.dataadministration.DataAdministrationModule;
import ibspan.ecap.merchant.shopdecisions.dataadministration.DeliveryLog;
import jade.core.Agent;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;
import jade.util.Logger;

public class RecordDeliveriesHandler extends OneShotBehaviour
{
	public static final String MESSAGE_KEY = "MessageKey";
	protected Logger myLogger = Logger.getMyLogger(this.getClass().getName());
	
	private Hashtable<String, Integer> orderProblemCounters;
		
	public RecordDeliveriesHandler(Agent myAgent)
	{
		super(myAgent);
	}

	public RecordDeliveriesHandler(Agent myAgent,
			Hashtable<String, Integer> orderProblemCounters) {
		this(myAgent);
		this.orderProblemCounters = orderProblemCounters;
	}

	@Override
	public void action()
	{
		ACLMessage message = (ACLMessage) getDataStore().get(RecordDeliveriesHandler.MESSAGE_KEY);
		
		if(message == null)
		{
			myLogger.warning("RecordDeliveriesHandler didn't receive a message"); 
			return;
		}
		DeliveryLog log;
		try
		{
			log = retrieveLog(message);
		} catch (UnreadableException e)
		{
			myLogger.warning("RecordDeliveriesHandler couldn't retrieve content from the message"); 
			return;
		} catch (ClassCastException e)
		{
			myLogger.warning("RecordDeliveriesHandler received wrong content in the message"); 
			return;
		}
		//Clear order problems counter
		if(orderProblemCounters != null)
		{
			Integer counter = orderProblemCounters.get(log.getDeliveredProductId());
			if(counter!=null)
				orderProblemCounters.remove(log.getDeliveredProductId());
		}
		
		//Record data to the database
		try
		{
			DataAdministrationModule.getInstance().recordWarehouseDeliveryDetails(log);
		} catch (DataAccessException e)
		{
			myLogger.warning("RecordDeliveriesHandler failed to record negotiation data."); 
			return;
		}		
	}

	private DeliveryLog retrieveLog(ACLMessage message) throws UnreadableException
	{
		DeliveryLog log = (DeliveryLog) message.getContentObject();
		return log;
	}

}
