package ibspan.ecap.merchant.shopdecisions;

import java.util.logging.Level;

import ibspan.ecap.merchant.shopdecisions.forecasting.ForecastingModule;
import jade.core.Agent;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;
import jade.util.Logger;

/**
 * A behaviour handling the request for stopping the sales forecast of a product. It does so by
 * withdrawing the product from periodical forecasting and cancelling a scheduled forecast if
 * it exists.
 * 
 * @author Micha Drozdowicz
 *
 */
public class StopForecastingHandler extends OneShotBehaviour {

	protected Logger myLogger = Logger.getMyLogger(this.getClass().getName());
	
	public static final String MESSAGE_KEY = "MessageKey";

	public StopForecastingHandler(Agent myAgent)
	{
		super(myAgent);
	}
	
	@Override
	public void action() {
		ACLMessage message = (ACLMessage) getDataStore().get(StopForecastingHandler.MESSAGE_KEY);
				
		if(message == null)
		{
			myLogger.warning("StopForecasting didn't receive a message"); 
			return;
		}
		
		String productId = null;		
		try {
			productId = retrieveProductId(message);
		} catch (UnreadableException e) {
			if(myLogger.isLoggable(Level.WARNING))
				myLogger.warning("StopForecasting couldn't read the message. Exception: "+e.toString());
			return;
		}
		
		ForecastingModule module = ForecastingModule.getInstance();
		if(myLogger.isLoggable(Level.INFO))
			myLogger.info("Stopping forecasting of product: "+productId);
		module.stopForecasting(productId);
	}

	protected String retrieveProductId(ACLMessage message) throws UnreadableException {
		return (String) message.getContentObject();
	}
}
