package ibspan.ecap.merchant.shopdecisions.dataaccess;

import ibspan.ecap.merchant.shopdecisions.dataadministration.ForecastFact;
import ibspan.ecap.shared.util.PropertiesProvider;

import jade.util.Logger;

import java.beans.PropertyVetoException;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Date;
import java.util.logging.Level;

import com.mchange.v2.c3p0.ComboPooledDataSource;
import com.mchange.v2.c3p0.DataSources;

/**
 * A service responsible for inserting and modifying the data in the data warehouse. Singleton.
 * @author  Micha Drozdowicz
 */
public class DataModificationService {
	
	private static final String DB_URL_PROP_NAME = "dataModification.dbUrl";

	private static final String DEF_DB_URL = "jdbc:postgresql://localhost/SDA_KB";

	private static final String DB_USER_PROP_NAME = "dataModification.dbUser";

	private static final String DEF_DB_USER = "postgres";

	private static final String DEF_DB_PASSWORD = "password";

	private static final String DB_PASSWORD_PROP_NAME = "dataModification.dbPassword";

	private static final String DEF_DB_DRIVER = "org.postgresql.Driver";

	private static final String DB_DRIVER_PROP_NAME = "dataModification.dbDriver";
	private ComboPooledDataSource dataSource;
		
	private static DataModificationService instance = new DataModificationService();
	
	private Logger myLogger = Logger.getMyLogger(this.getClass().getName());
	
	private DataModificationService()
	{
		PropertiesProvider propsProvider = PropertiesProvider.getInstance();
		String dbUrl = propsProvider.getStringProperty(DB_URL_PROP_NAME, DEF_DB_URL);
		String dbUser = propsProvider.getStringProperty(DB_USER_PROP_NAME, DEF_DB_USER);
		String dbPassword = propsProvider.getStringProperty(DB_PASSWORD_PROP_NAME, DEF_DB_PASSWORD);
		String dbDriver = propsProvider.getStringProperty(DB_DRIVER_PROP_NAME, DEF_DB_DRIVER);
				
		dataSource = new ComboPooledDataSource();
		try
		{
			dataSource.setDriverClass(dbDriver);
		} catch (PropertyVetoException e)
		{
			myLogger.log(Level.SEVERE, "Error setting the jdbc driver class.", e);
			throw new RuntimeException("Error setting the jdbc driver class.");
		} // loads the jdbc driver
		dataSource.setJdbcUrl(dbUrl);
		dataSource.setUser(dbUser); 
		dataSource.setPassword(dbPassword); 
	}
	
	@Override
	protected void finalize() throws Throwable
	{		
		DataSources.destroy( dataSource );
	}
	
	
	/**
	 * Gets the instance of the <code>DataAccessModule</code>.
	 *
	 * @return the instance
	 */
	public static DataModificationService getInstance()
	{
		return instance;
	}
	
	public int getDateKey(Date date) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement selectDate = conn
				.prepareCall("{?= call sp_ddt_getdatekey(?)}");
		selectDate.registerOutParameter(1, Types.INTEGER);
		selectDate.setDate(2, new java.sql.Date(date.getTime()));

		try
		{
			selectDate.execute();
			return selectDate.getInt(1);			
		} finally
		{
			selectDate.close();
			conn.close();
		}
	}
	
	public int insertDate(DateEntity date) throws SQLException
	{		
		Connection conn = getConnection();
		CallableStatement insertDate = conn
				.prepareCall("{?= call sp_ddt_insertdate(?,?,?,?,?,?,?,?,?,?,?,?,?)}");
		insertDate.registerOutParameter(1, Types.INTEGER);
		insertDate.setDate(2, new java.sql.Date(date.getFullDate().getTime()));
		insertDate.setString(3, date.getDayOfWeek());
		insertDate.setShort(4, (short) date.getDayNoInWeek());
		insertDate.setInt(5, date.getDayInEpoch());
		insertDate.setInt(6, date.getWeekInEpoch());
		insertDate.setInt(7, date.getMonthInEpoch());
		insertDate.setShort(8, (short) date.getDayInYear());
		insertDate.setShort(9, (short) date.getWeekInYear());
		insertDate.setShort(10, (short) date.getMonthInYear());
		insertDate.setShort(11, (short) date.getQuarterInYear());
		insertDate.setInt(12, date.getYear());
		insertDate.setShort(13, (short) date.getDayInMonth());
		insertDate.setString(14, date.getWeekdayIndicator());
		
		try
		{
			insertDate.execute();
			return insertDate.getInt(1);
		}
		finally
		{
			insertDate.close();
			conn.close();
		}
	}	
	
	public int getTimeKey(Date time) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement selectTime = conn
				.prepareCall("{?= call sp_dtm_gettimekey(?)}");
		selectTime.registerOutParameter(1, Types.INTEGER);
		selectTime.setTime(2, new java.sql.Time(time.getTime()));

		try
		{
			selectTime.execute();
			return selectTime.getInt(1);			
		} finally
		{
			selectTime.close();
			conn.close();
		}
	}
	
	public int insertTime(TimeEntity time) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement insertTime = conn
				.prepareCall("{?= call sp_dtm_inserttime(?,?,?)}");
		insertTime.registerOutParameter(1, Types.INTEGER);
		insertTime.setTime(2, new java.sql.Time(time.getFullTime().getTime()));
		insertTime.setShort(3, (short) time.getHour());
		insertTime.setString(4, time.getAmPmIndicator());
		
		try
		{
			insertTime.execute();
			return insertTime.getInt(1);
		}
		finally
		{
			insertTime.close();
			conn.close();
		}
	}
	
	public int getProductKey(String productId) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement selectProduct = conn
				.prepareCall("{?= call sp_dpd_getproductkey(?)}");
		selectProduct.registerOutParameter(1, Types.INTEGER);
		selectProduct.setString(2, productId);

		try
		{
			selectProduct.execute();
			return selectProduct.getInt(1);			
		} finally
		{
			selectProduct.close();
			conn.close();
		}
	}
	
	public int insertProduct(ProductEntity product) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement insertProduct = conn
				.prepareCall("{?= call sp_dpd_insertproduct(?)}");
		insertProduct.registerOutParameter(1, Types.INTEGER);
		insertProduct.setString(2, product.getGlobalId());
		
		try
		{
			insertProduct.execute();
			return insertProduct.getInt(1);
		}
		finally
		{
			insertProduct.close();
			conn.close();
		}
	}
	
	public int getWholesalerKey(String globalId) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement selectWholesaler = conn
				.prepareCall("{?= call sp_dwh_getwholesalerkey(?)}");
		selectWholesaler.registerOutParameter(1, Types.INTEGER);
		selectWholesaler.setString(2, globalId);

		try
		{
			selectWholesaler.execute();
			return selectWholesaler.getInt(1);			
		} finally
		{
			selectWholesaler.close();
			conn.close();
		}
	}
	
	public int insertWholesaler(WholesalerEntity entity) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement insertWholesaler = conn
				.prepareCall("{?= call sp_dwh_insertwholesaler(?)}");
		insertWholesaler.registerOutParameter(1, Types.INTEGER);
		insertWholesaler.setString(2, entity.getGlobalId());
		
		try
		{
			insertWholesaler.execute();
			return insertWholesaler.getInt(1);
		}
		finally
		{
			insertWholesaler.close();
			conn.close();
		}
	}
	
	public int getStrategyKey(String globalId) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement selectStrategy = conn
				.prepareCall("{?= call sp_dst_getstrategykey(?)}");
		selectStrategy.registerOutParameter(1, Types.INTEGER);
		selectStrategy.setString(2, globalId);

		try
		{
			selectStrategy.execute();
			return selectStrategy.getInt(1);			
		} finally
		{
			selectStrategy.close();
			conn.close();
		}
	}
	
	public int insertStrategy(StrategyEntity entity) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement insertStrategy = conn
				.prepareCall("{?= call sp_dst_insertstrategy(?,?)}");
		insertStrategy.registerOutParameter(1, Types.INTEGER);
		insertStrategy.setString(2, entity.getGlobalId());
		insertStrategy.setString(3, entity.getDescription());
		
		try
		{
			insertStrategy.execute();
			return insertStrategy.getInt(1);
		}
		finally
		{
			insertStrategy.close();
			conn.close();
		}
	}
	
	public int getTemplateKey(String globalId) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement selectTemplate = conn
				.prepareCall("{?= call sp_dtmp_gettemplatekey(?)}");
		selectTemplate.registerOutParameter(1, Types.INTEGER);
		selectTemplate.setString(2, globalId);

		try
		{
			selectTemplate.execute();
			return selectTemplate.getInt(1);			
		} finally
		{
			selectTemplate.close();
			conn.close();
		}
	}
	
	public int insertTemplate(TemplateEntity entity) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement insertTemplate = conn
				.prepareCall("{?= call sp_dtmp_inserttemplate(?,?)}");
		insertTemplate.registerOutParameter(1, Types.INTEGER);
		insertTemplate.setString(2, entity.getGlobalId());
		insertTemplate.setString(3, entity.getDescription());
		
		try
		{
			insertTemplate.execute();
			return insertTemplate.getInt(1);
		}
		finally
		{
			insertTemplate.close();
			conn.close();
		}
	}
	
	public SalesFact getSalesFact(int negotiationId) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement selectSalesFact = conn
				.prepareCall("{call sp_fsa_getsalesfact(?)}");
		
		selectSalesFact.setInt(1, negotiationId);
		try
		{
			ResultSet rs = selectSalesFact.executeQuery();
			if(rs.next())
			{
				SalesFact fact = new SalesFact();
				fact.setBidCount(rs.getInt("bidCount"));
				fact.setFinishedTransactionCount(rs.getInt("finishedTransactionCount"));
				fact.setMinimumValueOffered(rs.getFloat("minValueOffered"));
				fact.setNegotiationEndDateKey(rs.getInt("negotiationEndDateId"));
				fact.setNegotiationEndTimeKey(rs.getInt("negotiationEndTimeId"));
				fact.setNegotiationId(rs.getInt("negotiationId"));
				fact.setProductOfferedKey(rs.getInt("productId"));
				fact.setQuantityConfirmed(rs.getInt("quantityConfirmed"));
				fact.setQuantityOffered(rs.getInt("quantityOffered"));
				fact.setQuantityReserved(rs.getInt("quantityReserved"));
				fact.setStrategyUsedKey(rs.getInt("strategyId"));
				fact.setTemplateUsedKey(rs.getInt("templateId"));
				fact.setValueConfirmed(rs.getFloat("valueConfirmed"));
				fact.setValueOffered(rs.getFloat("valueOffered"));
				fact.setValueReserved(rs.getFloat("valueReserved"));
				fact.setWinningBidCount(rs.getInt("winningBidCount"));
				
				return fact;
			}
			else
				return null;
		} finally
		{
			selectSalesFact.close();
			conn.close();
		}
	}
	
	public void insertSalesFact(SalesFact fact) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement insertSalesFact = conn
				.prepareCall("{call sp_fsa_insertsalesfact(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)}");
	
		insertSalesFact.setInt(1, fact.getNegotiationEndDateKey());
		insertSalesFact.setInt(2, fact.getNegotiationEndTimeKey());
		insertSalesFact.setInt(3, fact.getProductOfferedKey());
		insertSalesFact.setInt(4, fact.getStrategyUsedKey());
		insertSalesFact.setInt(5, fact.getTemplateUsedKey());
		insertSalesFact.setInt(6, fact.getNegotiationId());
		insertSalesFact.setInt(7, fact.getQuantityOffered());
		insertSalesFact.setFloat(8, fact.getValueOffered());
		insertSalesFact.setFloat(9, fact.getMinimumValueOffered());
		insertSalesFact.setInt(10, fact.getQuantityReserved());
		insertSalesFact.setFloat(11, fact.getValueReserved());
		insertSalesFact.setInt(12, fact.getQuantityConfirmed());
		insertSalesFact.setFloat(13, fact.getValueConfirmed());
		insertSalesFact.setInt(14, fact.getBidCount());
		insertSalesFact.setInt(15, fact.getWinningBidCount());
		insertSalesFact.setInt(16, fact.getFinishedTransactionCount());
		try
		{
			insertSalesFact.execute();
		} finally
		{
			insertSalesFact.close();
			conn.close();
		}
	}
	
	public void updateSalesFact(SalesFact fact) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement insertSalesFact = conn
				.prepareCall("{call sp_fsa_updatesalesfact(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)}");
	
		insertSalesFact.setInt(1, fact.getNegotiationEndDateKey());
		insertSalesFact.setInt(2, fact.getNegotiationEndTimeKey());
		insertSalesFact.setInt(3, fact.getProductOfferedKey());
		insertSalesFact.setInt(4, fact.getStrategyUsedKey());
		insertSalesFact.setInt(5, fact.getTemplateUsedKey());
		insertSalesFact.setInt(6, fact.getNegotiationId());
		insertSalesFact.setInt(7, fact.getQuantityOffered());
		insertSalesFact.setFloat(8, fact.getValueOffered());
		insertSalesFact.setFloat(9, fact.getMinimumValueOffered());
		insertSalesFact.setInt(10, fact.getQuantityReserved());
		insertSalesFact.setFloat(11, fact.getValueReserved());
		insertSalesFact.setInt(12, fact.getQuantityConfirmed());
		insertSalesFact.setFloat(13, fact.getValueConfirmed());
		insertSalesFact.setInt(14, fact.getBidCount());
		insertSalesFact.setInt(15, fact.getWinningBidCount());
		insertSalesFact.setInt(16, fact.getFinishedTransactionCount());
		try
		{
			insertSalesFact.execute();
		} finally
		{
			insertSalesFact.close();
			conn.close();
		}
	}
	
	public SuppliesFact getSuppliesFact(int wholesalerKey,	int productKey, int orderDateKey, int orderTimeKey, int deliveryDateKey, int deliveryTimeKey) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement selectSuppliesFact = conn
				.prepareCall("{call sp_fsp_getsuppliesfact(?, ?, ?, ?, ?, ?)}");
		
		selectSuppliesFact.setInt(1, wholesalerKey);
		selectSuppliesFact.setInt(2, productKey);
		selectSuppliesFact.setInt(3, orderDateKey);
		selectSuppliesFact.setInt(4, orderTimeKey);
		selectSuppliesFact.setInt(5, deliveryDateKey);
		selectSuppliesFact.setInt(6, deliveryTimeKey);
		
		try
		{
			ResultSet rs = selectSuppliesFact.executeQuery();
			if(rs.next())
			{
				SuppliesFact fact = new SuppliesFact();
				fact.setDeliveryDateKey(rs.getInt("wholesalerId"));
				fact.setProductKey(rs.getInt("productId"));
				fact.setOrderDateKey(rs.getInt("orderDateId"));
				fact.setOrderTimeKey(rs.getInt("orderTimeId"));
				fact.setDeliveryDateKey(rs.getInt("deliveryDateId"));				
				fact.setDeliveryTimeKey(rs.getInt("deliveryTimeId"));
				fact.setQuantityOrdered(rs.getInt("quantityOrdered"));
				fact.setValueOrdered(rs.getFloat("valueOrdered"));
				fact.setQuantityDelivered(rs.getInt("quantityDelivered"));
				fact.setValueDelivered(rs.getFloat("valueDelivered"));
								
				return fact;
			}
			else
				return null;
		} finally
		{
			selectSuppliesFact.close();
			conn.close();
		}
	}
	
	public void insertSuppliesFact(SuppliesFact fact) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement insertSuppliesFact = conn
				.prepareCall("{call sp_fsp_insertsuppliesfact(?,?,?,?,?,?,?,?,?,?)}");
//		wholesalerId integer, productId integer, orderDateId integer, orderTimeId integer, 
//		  deliveryDateId integer, deliveryTimeId integer, quantityOrdered integer, valueOrdered numeric(12,2), quantityDelivered integer, 
//		  valueDelivered numeric(12,2)
		
		insertSuppliesFact.setInt(1, fact.getWholesalerKey());
		insertSuppliesFact.setInt(2, fact.getProductKey());
		insertSuppliesFact.setInt(3, fact.getOrderDateKey());
		insertSuppliesFact.setInt(4, fact.getOrderTimeKey());
		insertSuppliesFact.setInt(5, fact.getDeliveryDateKey());
		insertSuppliesFact.setInt(6, fact.getDeliveryTimeKey());
		insertSuppliesFact.setInt(7, fact.getQuantityOrdered());
		insertSuppliesFact.setFloat(8, fact.getValueOrdered());
		insertSuppliesFact.setInt(9, fact.getQuantityDelivered());
		insertSuppliesFact.setFloat(10, fact.getValueDelivered());
		try
		{
			insertSuppliesFact.execute();
		} finally
		{
			insertSuppliesFact.close();
			conn.close();
		}
	}
	
	public DemandFact getDemandFact(int dateKey, int productKey) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement selectDemandFact = conn
				.prepareCall("{call sp_fdm_getdemandfact(?, ?)}");
		
		selectDemandFact.setInt(1, dateKey);
		selectDemandFact.setInt(2, productKey);
		
		try
		{
			ResultSet rs = selectDemandFact.executeQuery();
			if(rs.next())
			{
				DemandFact fact = new DemandFact();
				fact.setDateKey(rs.getInt("dateId"));
				fact.setProductKey(rs.getInt("productId"));
				fact.setCicQueryCount(rs.getInt("cicQueryCount"));
				fact.setShopsSellingCount(rs.getInt("shopsSellingCount"));
				fact.setRegisteredClients(rs.getInt("registeredClients"));
								
				return fact;
			}
			else
				return null;
		} finally
		{
			selectDemandFact.close();
			conn.close();
		}
	}
	
	public void insertDemandFact(DemandFact fact) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement insertDemandFact = conn
				.prepareCall("{call sp_fdm_insertdemandfact(?,?,?,?,?)}");

		insertDemandFact.setInt(1, fact.getDateKey());
		insertDemandFact.setInt(2, fact.getProductKey());
		insertDemandFact.setInt(3, fact.getCicQueryCount());
		insertDemandFact.setInt(4, fact.getShopsSellingCount());
		insertDemandFact.setInt(5, fact.getRegisteredClients());
		try
		{
			insertDemandFact.execute();
		} finally
		{
			insertDemandFact.close();
			conn.close();
		}
	}
	
	public void updateDemandFact(DemandFact fact) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement updateDemandFact = conn
				.prepareCall("{call sp_fdm_updatedemandfact(?,?,?,?,?)}");

		updateDemandFact.setInt(1, fact.getDateKey());
		updateDemandFact.setInt(2, fact.getProductKey());
		updateDemandFact.setInt(3, fact.getCicQueryCount());
		updateDemandFact.setInt(4, fact.getShopsSellingCount());
		updateDemandFact.setInt(5, fact.getRegisteredClients());
		try
		{
			updateDemandFact.execute();
		} finally
		{
			updateDemandFact.close();
			conn.close();
		}
	}
	
	public void insertForecastFact(ForecastFact fact) throws SQLException
	{
		Connection conn = getConnection();
		CallableStatement insertForecastFact = conn
				.prepareCall("{call sp_fsf_insertforecastfact(?,?,?,?,?)}");

		insertForecastFact.setInt(1, fact.getStartDateKey());
		insertForecastFact.setInt(2, fact.getEndDateKey());
		insertForecastFact.setInt(3, fact.getProductKey());
		insertForecastFact.setFloat(4, fact.getQuantity());
		insertForecastFact.setFloat(5, fact.getDeviation());
		try
		{
			insertForecastFact.execute();
		} finally
		{
			insertForecastFact.close();
			conn.close();
		}
	}
	
	private Connection getConnection() throws SQLException
	{
		return dataSource.getConnection();
	}

	
}
