package ibspan.ecap.merchant.shopdecisions.dataaccess;

import ibspan.ecap.shared.util.PropertiesProvider;
import jade.util.Logger;

import java.beans.PropertyVetoException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.Level;

import com.mchange.v2.c3p0.ComboPooledDataSource;
import com.mchange.v2.c3p0.DataSources;

/**
 * A service responsible for accessing the data warehouse and retrieving the knowledge contained within it. Singleton.
 *
 * @author Micha Drozdowicz
 *
 */
public class DataQueryService
{
	private static final String DB_URL_PROP_NAME = "dataQuery.dbUrl";
	private static final String DEF_DB_URL = "jdbc:postgresql://localhost/SDA_KB";
	private static final String DB_USER_PROP_NAME = "dataQuery.dbUser";
	private static final String DEF_DB_USER = "postgres";
	private static final String DEF_DB_PASSWORD = "password";
	private static final String DB_PASSWORD_PROP_NAME = "dataQuery.dbPassword";
	private static final String DEF_DB_DRIVER = "org.postgresql.Driver";
	private static final String DB_DRIVER_PROP_NAME = "dataQuery.dbDriver";
	
	private Logger myLogger = Logger.getMyLogger(this.getClass().getName());
				
	private ComboPooledDataSource dataSource;
		
	private static DataQueryService instance = new DataQueryService();

	private DataQueryService()
	{
		PropertiesProvider propsProvider = PropertiesProvider.getInstance();
		String dbUrl = propsProvider.getStringProperty(DB_URL_PROP_NAME, DEF_DB_URL);
		String dbUser = propsProvider.getStringProperty(DB_USER_PROP_NAME, DEF_DB_USER);
		String dbPassword = propsProvider.getStringProperty(DB_PASSWORD_PROP_NAME, DEF_DB_PASSWORD);
		String dbDriver = propsProvider.getStringProperty(DB_DRIVER_PROP_NAME, DEF_DB_DRIVER);
		
		dataSource = new ComboPooledDataSource();
		try
		{
			dataSource.setDriverClass(dbDriver);
		} catch (PropertyVetoException e)
		{
			myLogger.log(Level.SEVERE, "Error setting the jdbc driver class.", e);
			throw new RuntimeException("Error setting the jdbc driver class.");
		} // loads the jdbc driver
		dataSource.setJdbcUrl(dbUrl);
		dataSource.setUser(dbUser); 
		dataSource.setPassword(dbPassword); 
	}
	
	@Override
	protected void finalize() throws Throwable
	{		
		DataSources.destroy( dataSource );
	}
	
	
	/**
	 * Gets the instance of the <code>DataQueryService</code>.
	 *
	 * @return the instance
	 */
	public static DataQueryService getInstance()
	{
		return instance;
	}
	
	public void executeQuery(String queryText, ResultProcessor processor) throws SQLException
	{
		Connection conn = getConnection();
		Statement query = conn.createStatement();

		if(myLogger.isLoggable(Logger.FINER))
			myLogger.finer("Query text: "+queryText);
		
		try
		{			
			ResultSet rs = query.executeQuery(queryText);
			processor.processResult(rs);			
		}
		finally
		{
			
			query.close();
			conn.close();
		}		
	}
	
	private Connection getConnection() throws SQLException
	{
		return dataSource.getConnection();
	}
	
	
}
