package ibspan.ecap.merchant.shopdecisions.dataaccess;

import java.util.Date;

/**
 * A class representing a date dimension entity.
 * @author  Micha Drozdowicz
 */
public class DateEntity
{
	private Date fullDate;
	private String dayOfWeek;
	private int dayNoInWeek;
	private int dayInEpoch;
	private int weekInEpoch;
	private int monthInEpoch;
	private int dayInYear;
	private int weekInYear;
	private int monthInYear;
	private int quarterInYear;
	private int year;
	private int dayInMonth;
	private String weekdayIndicator;

	/**
	 * Creates a new instance of <code>DateEntity</code>. 
	 * 
	 * @param fullDate
	 *            the full date.
	 * @param dayOfWeek
	 *            the name of the day of week. Can equal any of the following:
	 *            Monday Tuesday Wednesday Thursday Friday Saturday Sunday
	 * @param dayNoInWeek
	 *            the number of the day in a week.
	 * @param dayInEpoch
	 *            the number of the day in epoch (since 01.01.1970).
	 * @param weekInEpoch
	 *            the number of the week in epoch (since 01.01.1970).
	 * @param monthInEpoch
	 *            the number of the month in epoch (since 01.01.1970).
	 * @param dayInYear
	 *            the number of the day in year.
	 * @param weekInYear
	 *            the number of the week in year.
	 * @param monthInYear
	 *            the number of the month in year.
	 * @param quarterInYear
	 *            the number of the quarter in year.
	 * @param year
	 *            the year number AD.
	 * @param dayInMonth
	 *            the day number in month.
	 * @param weekdayIndicator
	 *            the weekday indicator. Takes a value of 'Weekday' or
	 *            'Weekend'.
	 */
	public DateEntity(Date fullDate, String dayOfWeek, int dayNoInWeek, int dayInEpoch, int weekInEpoch, int monthInEpoch, int dayInYear, int weekInYear, int monthInYear, int quarterInYear, int year, int dayInMonth, String weekdayIndicator)
	{
		this.fullDate = fullDate;
		this.dayOfWeek = dayOfWeek;
		this.dayNoInWeek = dayNoInWeek;
		this.dayInEpoch = dayInEpoch;
		this.weekInEpoch = weekInEpoch;
		this.monthInEpoch = monthInEpoch;
		this.dayInYear = dayInYear;
		this.weekInYear = weekInYear;
		this.monthInYear = monthInYear;
		this.quarterInYear = quarterInYear;
		this.year = year;
		this.dayInMonth = dayInMonth;
		this.weekdayIndicator = weekdayIndicator;
	}

	/**
	 * Gets the number of day in epoch (since 01.01.1970).
	 *
	 * @return the number of day in epoch (since 01.01.1970).
	 */
	public int getDayInEpoch()
	{
		return dayInEpoch;
	}

	/**
	 * Gets the day number in month
	 *
	 * @return the day number in month.
	 */
	public int getDayInMonth()
	{
		return dayInMonth;
	}

	/**
	 * Gets the number of the day in year.
	 *
	 * @return the number of the day in year.
	 */
	public int getDayInYear()
	{
		return dayInYear;
	}

	/**
	 * Gets the number of day in a week (1-7).
	 *
	 * @return the number of day in a week (1-7).
	 */
	public int getDayNoInWeek()
	{
		return dayNoInWeek;
	}

	/**
	 * Gets the name of the day of week. 
	 * Can equal any of the following:
	 * 
	 * Monday Tuesday Wednesday Thursday Friday Saturday Sunday
	 *
	 * @return the name of the day of week.
	 */
	public String getDayOfWeek()
	{
		return dayOfWeek;
	}

	/**
	 * Gets the full date.
	 *
	 * @return the full date.
	 */
	public Date getFullDate()
	{
		return fullDate;
	}

	/**
	 * Gets the number of the month in epoch (since 01.01.1970).
	 *
	 * @return the number of the month in epoch (since 01.01.1970)
	 */
	public int getMonthInEpoch()
	{
		return monthInEpoch;
	}

	/**
	 * Gets the number of the month in year.
	 *
	 * @return the number of the month in year.
	 */
	public int getMonthInYear()
	{
		return monthInYear;
	}

	/**
	 * Gets the quarter number in year.
	 *
	 * @return the quarter number in year.
	 */
	public int getQuarterInYear()
	{
		return quarterInYear;
	}

	/**
	 * Gets the weekday indicator. Takes a value of 'Weekday' or 'Weekend'.
	 *
	 * @return the weekday indicator.
	 */
	public String getWeekdayIndicator()
	{
		return weekdayIndicator;
	}

	/**
	 * Gets the number of week in epoch (since 01.01.1970).
	 *
	 * @return returns the number of the week since 01.01.1970
	 */
	public int getWeekInEpoch()
	{
		return weekInEpoch;
	}

	/**
	 * Get the week number in year.
	 *
	 * @return the week number in year.
	 */
	public int getWeekInYear()
	{
		return weekInYear;
	}

	/**
	 * Gets the year number AD.
	 *
	 * @return the year number AD.
	 */
	public int getYear()
	{
		return year;
	}

}
