package ibspan.ecap.merchant.shopdecisions.dataaccess;

/**
 * A class representing an order of product from a wholesaler.  It is roughly equivalent to an entity of the Supplies fact table.
 * @author  Micha Drozdowicz
 */
public class SuppliesFact
{
	private int wholesalerKey;
	private int productKey;
	private int orderDateKey;
	private int orderTimeKey;
	private int deliveryDateKey;
	private int deliveryTimeKey;
	private int quantityOrdered;
	private float valueOrdered;
	private int quantityDelivered;
	private float valueDelivered;
	
	
	public SuppliesFact()
	{}


	/**
	 * Creates a new instance of <code>SuppliesFact</code>.
	 * 
	 * @param wholesalerKey the wholesaler key
	 * @param productKey the product key
	 * @param orderDateKey the order date key
	 * @param orderTimeKey the order time key
	 * @param deliveryDateKey the delivery date key
	 * @param deliveryTimeKey the delivery time key
	 * @param quantityOrdered the quantity ordered
	 * @param valueOrdered the value ordered
	 * @param quantityDelivered the quantity delivered
	 * @param valueDelivered the value delivered
	 */
	public SuppliesFact(int wholesalerKey, int productKey, int orderDateKey,
			int orderTimeKey, int deliveryDateKey, int deliveryTimeKey,
			int quantityOrdered, float valueOrdered, int quantityDelivered,
			float valueDelivered)
	{
		this.wholesalerKey = wholesalerKey;
		this.productKey = productKey;
		this.orderDateKey = orderDateKey;
		this.orderTimeKey = orderTimeKey;
		this.deliveryDateKey = deliveryDateKey;
		this.deliveryTimeKey = deliveryTimeKey;
		this.quantityOrdered = quantityOrdered;
		this.valueOrdered = valueOrdered;
		this.quantityDelivered = quantityDelivered;
		this.valueDelivered = valueDelivered;
	}


	/**
	 * Gets the key of the wholesaler dimension entry describing the wholesaler who handled this order.
	 * 
	 * @return key id of the wholesaler dimension entry describing the wholesaler who handled this order.
	 */
	public int getWholesalerKey()
	{
		return wholesalerKey;
	}


	/**
	 * Sets the id of the wholesaler dimension entry describing the wholesaler who handled this order.
	 *
	 * @param wholesalerKey the id of the wholesaler dimension entry describing the wholesaler who handled this order.
	 */
	public void setWholesalerKey(int wholesalerKey)
	{
		this.wholesalerKey = wholesalerKey;
	}


	/**
	 * Gets the key to the product dimension entry describing the product delivered.
	 *
	 * @return the key to the product dimension entry describing the product delivered.
	 */
	public int getProductKey()
	{
		return productKey;
	}


	/**
	 * Sets the key to the product dimension entry describing the product delivered.
	 *
	 * @param productKey key to the product dimension entry describing the product delivered.
	 */
	public void setProductKey(int productKey)
	{
		this.productKey = productKey;
	}


	/**
	 * Gets the key to the date dimension entry describing the date of the making of the order.
	 *
	 * @return the key to the date dimension entry describing the date of the making of the order.
	 */
	public int getOrderDateKey()
	{
		return orderDateKey;
	}


	/**
	 * Sets the key to the date dimension entry describing the date of the making of the order.
	 *
	 * @param orderDateKey the key to the date dimension entry describing the date of the making of the order.
	 */
	public void setOrderDateKey(int orderDateKey)
	{
		this.orderDateKey = orderDateKey;
	}


	/**
	 * Gets the key to the time dimension entry describing the time of day of the making of the order.
	 *
	 * @return the key to the time dimension entry describing the time of day of the making of the order.
	 */
	public int getOrderTimeKey()
	{
		return orderTimeKey;
	}


	/**
	 * Sets the key to the time dimension entry describing the time of day of the making of the order.
	 *
	 * @param orderTimeKey the key to the time dimension entry describing the time of day of the making of the order.
	 */
	public void setOrderTimeKey(int orderTimeKey)
	{
		this.orderTimeKey = orderTimeKey;
	}


	/**
	 * Gets the key to the date dimension entry describing the date the ordered product was delivered.
	 *
	 * @return the key to the date dimension entry describing the date the ordered product was delivered.
	 */
	public int getDeliveryDateKey()
	{
		return deliveryDateKey;
	}


	/**
	 * Sets the key to the date dimension entry describing the date the ordered product was delivered.
	 *
	 * @param deliveryDateKey the key to the date dimension entry describing the date the ordered product was delivered.
	 */
	public void setDeliveryDateKey(int deliveryDateKey)
	{
		this.deliveryDateKey = deliveryDateKey;
	}


	/**
	 * Gets the key to the time dimension entry describing the time of day when the ordered product was delivered.
	 *
	 * @return the key to the time dimension entry describing the time of day when the ordered product was delivered.
	 */
	public int getDeliveryTimeKey()
	{
		return deliveryTimeKey;
	}


	/**
	 * Sets the key to the time dimension entry describing the time of day when the ordered product was delivered.
	 *
	 * @param deliveryTimeKey the key to the time dimension entry describing the time of day when the ordered product was delivered.
	 */
	public void setDeliveryTimeKey(int deliveryTimeKey)
	{
		this.deliveryTimeKey = deliveryTimeKey;
	}


	/**
	 * Gets the amount of product ordered
	 *
	 * @return the amount of product ordered
	 */
	public int getQuantityOrdered()
	{
		return quantityOrdered;
	}


	/**
	 * Sets the amount of product ordered
	 *
	 * @param quantityOrdered the amount of product ordered
	 */
	public void setQuantityOrdered(int quantityOrdered)
	{
		this.quantityOrdered = quantityOrdered;
	}


	/**
	 * Gets the total value of the product ordered using the ordering price
	 *
	 * @return the total value of the product ordered using the ordering price
	 */
	public float getValueOrdered()
	{
		return valueOrdered;
	}


	/**
	 * Sets the total value of the product ordered using the ordering price
	 *
	 * @param valueOrdered total value of the product ordered using the ordering price
	 */
	public void setValueOrdered(float valueOrdered)
	{
		this.valueOrdered = valueOrdered;
	}


	/**
	 * Gets the amount of product delivered by the wholesaler.
	 *
	 * @return the amount of product delivered by the wholesaler
	 */
	public int getQuantityDelivered()
	{
		return quantityDelivered;
	}


	/**
	 * Sets the amount of product delivered by the wholesaler.
	 *
	 * @param quantityDelivered the amount of product delivered by the wholesaler.
	 */
	public void setQuantityDelivered(int quantityDelivered)
	{
		this.quantityDelivered = quantityDelivered;
	}


	/**
	 * Gets the total value of the delivered product using the delivery price.
	 * 
	 * @return the total value of the delivered product using the delivery price.
	 */
	public float getValueDelivered()
	{
		return valueDelivered;
	}


	/**
	 * Sets the total value of the delivered product using the delivery price.
	 *
	 * @param valueDelivered the total value of the delivered product using the delivery price.
	 */
	public void setValueDelivered(float valueDelivered)
	{
		this.valueDelivered = valueDelivered;
	}
	
	
}
