package ibspan.ecap.merchant.shopdecisions.dataadministration;

import java.io.Serializable;
import java.util.Date;

/**
 * 
 * A class containing information about daily demand for a specific product.
 *
 * @author Micha Drozdowicz
 *
 */
public class DemandData implements Serializable
{
	private String productId;
	private Date date;
	private int cicQueryCount;
	private int shopsSellingProductCount;
	private int registeredClients;
	
	/**
	 * Gets the total number of clients that registered with the GA wanting to buy the product.
	 *
	 * @return the total number of clients that registered with the GA wanting to buy the product.
	 */
	public int getRegisteredClients()
	{
		return registeredClients;
	}
	/**
	 * Sets the total number of clients that registered with the GA wanting to buy the product.
	 *
	 * @param registeredClients total number of clients that registered with the GA wanting to buy the product.
	 */
	public void setRegisteredClients(int registeredClients)
	{
		this.registeredClients = registeredClients;
	}
	/**
	 * Gets the id of the product.
	 *
	 * @return the id of the product.
	 */
	public String getProductId()
	{
		return productId;
	}
	/**
	 * Sets the id of the product.
	 *
	 * @param productId the id of the product.
	 */
	public void setProductId(String productId)
	{
		this.productId = productId;
	}
	/**
	 * Gets the date on which the data was collected.
	 *
	 * @return the date
	 */
	public Date getDate()
	{
		return date;
	}
	/**
	 * Sets the date on which the data was collected.
	 *
	 * @param date the date to set
	 */
	public void setDate(Date date)
	{
		this.date = date;
	}
	/**
	 * Gets the amount of queries the CIC has registered during the day.
	 *
	 * @return the amount of CIC queries.
	 */
	public int getCicQueryCount()
	{
		return cicQueryCount;
	}
	/**
	 * Sets the amount of queries the CIC has registered during the day.
	 *
	 * @param cicQueryCount the amount of CIC queries.
	 */
	public void setCicQueryCount(int cicQueryCount)
	{
		this.cicQueryCount = cicQueryCount;
	}
	/**
	 * Gets the amount of shops selling the specified product during the day.
	 *
	 * @return the amount of shops selling the product
	 */
	public int getShopsSellingProductCount()
	{
		return shopsSellingProductCount;
	}
	/**
	 * Sets the amount of shops selling the specified product during the day.
	 *
	 * @param shopsSellingProductCount the amount of shops selling the product
	 */
	public void setShopsSellingProductCount(int shopsSellingProductCount)
	{
		this.shopsSellingProductCount = shopsSellingProductCount;
	}
	
	@Override
	public String toString() {
		return String.format("DemandData: {\n" +
				"\tproductId: %s\n" +
				"\tdate: %s\n" +
				"\tcicQueryCount: %s\n" +
				"\tshopsSellingProductCount: %s\n" +
				"\tregisteredClients: %s\n" +
				"}\n", 
				productId, date, cicQueryCount, shopsSellingProductCount, 
				registeredClients);
	}

}
