package ibspan.ecap.merchant.shopdecisions.dataadministration;

import ibspan.ecap.merchant.shopdecisions.dataaccess.DataModificationService;
import ibspan.ecap.merchant.shopdecisions.dataaccess.ProductEntity;

import java.sql.SQLException;

/**
 * A class responsible for handling and staging the product dimension.
 * 
 * @author Micha Drozdowicz
 */
public class ProductProcessor
{
	private DataModificationService dms;

	private static Object padlock = new Object();
	
	/**
	 * Instantiates a new product processor.
	 * 
	 * @param svc the {@link DataModificationService} to use to access the database
	 */
	public ProductProcessor(DataModificationService svc)
	{
		dms = svc;
	}

	/**
	 * Retrieve product key.
	 * 
	 * @param productId the id of the product, the key of which is to be retrieved.
	 * 
	 * @return the the key of the product
	 * 
	 * @throws SQLException the SQL exception
	 */
	public int retrieveProductKey(String productId) throws SQLException
	{
		synchronized (padlock) {
			int key = dms.getProductKey(productId);
			if (key > 0) {
				return key;
			} else {
				key = dms.insertProduct(createProductEntity(productId));
				return key;
			}
		}
	}

	private ProductEntity createProductEntity(String productId)
	{
		return new ProductEntity(productId);
	}
}
