package ibspan.ecap.merchant.shopdecisions.dataadministration;

import ibspan.ecap.merchant.shopdecisions.dataaccess.DataModificationService;
import ibspan.ecap.merchant.shopdecisions.dataaccess.StrategyEntity;

import java.sql.SQLException;

/**
 * A class responsible for handling and staging the strategy dimension.
 * 
 * @author Micha Drozdowicz
 */
public class StrategyProcessor
{
	private DataModificationService dms;
	private static Object padlock = new Object();
	
	/**
	 * Instantiates a new strategy processor.
	 * 
	 * @param svc the {@link DataModificationService} to use to access the database
	 */
	public StrategyProcessor(DataModificationService svc)
	{
		dms = svc;
	}

	/**
	 * Retrieve strategy key.
	 * 
	 * @param strategyId the id of the strategy, the key of which is to be retrieved.
	 * 
	 * @return the the key of the strategy
	 * 
	 * @throws SQLException the SQL exception
	 */
	public int retrieveStrategyKey(String strategyId) throws SQLException
	{
		synchronized (padlock) {
			int key = dms.getStrategyKey(strategyId);
			if (key > 0) {
				return key;
			} else {
				key = dms.insertStrategy(createStrategyEntity(strategyId));
				return key;
			}
		}		
	}

	/**
	 * Creates the strategy entity.
	 * 
	 * @param strategyId the strategy id
	 * 
	 * @return the strategy entity
	 */
	private StrategyEntity createStrategyEntity(String strategyId)
	{
		return new StrategyEntity(strategyId, "Unknown strategy");
	}
	
	
}
