package ibspan.ecap.merchant.shopdecisions.dataadministration;

import ibspan.ecap.merchant.shopdecisions.dataaccess.SuppliesFact;

/**
 * A class responsible for extracting the appropriate information from the supply delivery log and creating a supplies fact object.
 * 
 * @author Micha Drozdowicz
 */
public class SuppliesFactProcessor
{	
	
	/**
	 * Creates the supplies fact.
	 * 
	 * @param orderDateKey the key to the date dimension entry describing the date of the making of the order
	 * @param orderTimeKey the key to the time dimension entry describing the time of day of the making of the order.
	 * @param deliveryDateKey the key to the date dimension entry describing the date the ordered product was delivered.
	 * @param deliveryTimeKey the key to the time dimension entry describing the time of day when the ordered product was delivered.
	 * @param productKey the key to the product dimension entry describing the product delivered.
	 * @param wholesalerKey the key of the wholesaler dimension entry describing the wholesaler who handled this order.
	 * @param log the log describing the delivery
	 * 
	 * @return the supplies fact
	 */
	public SuppliesFact createSuppliesFact(int orderDateKey, int orderTimeKey,
			int deliveryDateKey, int deliveryTimeKey, int productKey,
			int wholesalerKey, DeliveryLog log)
	{
		SuppliesFact fact = new SuppliesFact();
		fact.setDeliveryDateKey(deliveryDateKey);
		fact.setDeliveryTimeKey(deliveryTimeKey);
		fact.setOrderDateKey(orderDateKey);
		fact.setOrderTimeKey(orderTimeKey);
		fact.setProductKey(productKey);
		fact.setWholesalerKey(wholesalerKey);
		fact.setQuantityDelivered(log.getOrderedQuantity());
		fact.setQuantityOrdered(log.getOrderedQuantity());
		fact.setValueDelivered(log.getOrderedValue());
		fact.setValueOrdered(log.getOrderedValue());
		
		return fact;
	}
}
