package ibspan.ecap.merchant.shopdecisions.dataadministration;

import ibspan.ecap.shared.ontology.negotiation.BidInfo;

import java.io.Serializable;
import java.util.Date;

public class TransactionLog implements Serializable
{
	private String transactionId;
	private int negotiationId;
	private String clientId;
	private Date time;
	private FinalisationType finalisationType;
	private int quantityReserved;
	private float valueReserved;
	
	/**
	 * Gets the amount of the product that was reserved.
	 *
	 * @return the amount of the product that was reserved.
	 */
	public int getQuantityReserved()
	{
		return quantityReserved;
	}
	/**
	 * Sets the amount of the product that was reserved.
	 *
	 * @param quantityReserved the amount of the product that was reserved.
	 */
	public void setQuantityReserved(int quantityReserved)
	{
		this.quantityReserved = quantityReserved;
	}
	/**
	 * Gets the value of the product that was reserved (at the price offered by the client).
	 *
	 * @return the value of the product that was reserved (at the price offered by the client).
	 */
	public float getValueReserved()
	{
		return valueReserved;
	}
	/**
	 * Sets the value of the product that was reserved (at the price offered by the client).
	 *
	 * @param valueReserved value of the product that was reserved (at the price offered by the client).
	 */
	public void setValueReserved(float valueReserved)
	{
		this.valueReserved = valueReserved;
	}
	/**
	 * Gets the id of the finished transaction.
	 *
	 * @return the id of the finished transaction
	 */
	public String getTransactionId()
	{
		return transactionId;
	}
	/**
	 * Sets the id of the finished transaction
	 *
	 * @param transactionId the id of the finished transaction
	 */
	public void setTransactionId(String transactionId)
	{
		this.transactionId = transactionId;
	}
	/**
	 * Gets the id of the client who made the reservation.
	 *
	 * @return the id of the client who made the reservation.
	 */
	public String getClientId()
	{
		return clientId;
	}
	/**
	 * Sets the id of the client who made the reservation.
	 *
	 * @param clientId the id of the client who made the reservation.
	 */
	public void setClientId(String clientId)
	{
		this.clientId = clientId;
	}
	/**
	 * Gets the time of the reservation finalisation.
	 *
	 * @return the time of the reservation finalisation.
	 */
	public Date getTime()
	{
		return time;
	}
	/**
	 * Sets the time of the reservation finalisation.
	 *
	 * @param time the time of the reservation finalisation.
	 */
	public void setTime(Date time)
	{
		this.time = time;
	}
	/**
	 * Gets the type of the finalisation.
	 *
	 * @return the type of the finalisation.
	 */
	public FinalisationType getFinalisationType()
	{
		return finalisationType;
	}
	/**
	 * Sets the type of the finalisation.
	 *
	 * @param finalisationType the type of the finalisation.
	 */
	public void setFinalisationType(FinalisationType finalisationType)
	{
		this.finalisationType = finalisationType;
	}
	/**
	 * Gets the id of the negotiation after which the reservation has been made.
	 *
	 * @return the id of the negotiation after which the reservation has been made.
	 */
	public int getNegotiationId()
	{
		return negotiationId;
	}
	/**
	 * Sets the id of the negotiation after which the reservation has been made.
	 *
	 * @param negotiationId the id of the negotiation after which the reservation has been made.
	 */
	public void setNegotiationId(int negotiationId)
	{
		this.negotiationId = negotiationId;
	}
	

	@Override
	public String toString() {
		return String.format("TransactionLog: {\n" +
				"\ttransactionId: %s\n" +
				"\tnegotiationId: %s\n" +
				"\tclientId: %s\n" +
				"\ttime: %s\n" +
				"\tfinalisationType: %s\n" +
				"\tquantityReserved: %s\n" +
				"}\n", 
				transactionId, negotiationId, clientId, time,
				finalisationType, quantityReserved, valueReserved);
	}
	
}
