package ibspan.ecap.merchant.shopdecisions.dataadministration;

import java.sql.SQLException;

import ibspan.ecap.merchant.shopdecisions.dataaccess.DataModificationService;
import ibspan.ecap.merchant.shopdecisions.dataaccess.WholesalerEntity;

/**
 * A class responsible for handling and staging the wholesaler dimension.
 * 
 * @author Micha Drozdowicz
 */
public class WholesalerProcessor
{
	private DataModificationService dms;
	private static Object padlock = new Object();
	
	/**
	 * Instantiates a new wholesaler processor.
	 * 
	 * @param svc the {@link DataModificationService} to use to access the database
	 */
	public WholesalerProcessor(DataModificationService svc)
	{
		dms = svc;
	}

	/**
	 * Retrieves wholesaler key.
	 * 
	 * @param wholesalerId the id of the wholesaler, the key of which is to be retrieved.
	 * 
	 * @return the key of the wholesaler dimension key
	 * 
	 * @throws SQLException the SQL exception
	 */
	public int retrieveWholesalerKey(String wholesalerId) throws SQLException
	{
		synchronized (padlock) {
			int key = dms.getWholesalerKey(wholesalerId);
			if (key > 0) {
				return key;
			} else {
				key = dms
						.insertWholesaler(createWholesalerEntity(wholesalerId));
				return key;
			}
		}		
	}

	/**
	 * Creates the wholesaler entity.
	 * 
	 * @param wholesalerId the wholesaler id
	 * 
	 * @return the wholesaler entity
	 */
	private WholesalerEntity createWholesalerEntity(String wholesalerId)
	{
		WholesalerEntity we = new WholesalerEntity(wholesalerId);
		return we;
	}

}
