package ibspan.ecap.merchant.shopdecisions.forecasting;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * A class managing the list of products which sales are forecasted.
 * @author  Michal
 */
public class ForecastedProductInventory
{
	private SortedSet<String> forecastedProducts;

	public ForecastedProductInventory()
	{
		forecastedProducts = new TreeSet<String>();
	}

	/**
	 * Registers a new product for sales forecasting.
	 * 
	 * @param productId
	 *            The id of the product to be forecasted.
	 */
	public void registerForForecast(String productId)
	{
		if (!forecastedProducts.contains(productId))
		{
			forecastedProducts.add(productId);
		}
	}

	/**
	 * Withdraws a product from sales forecasting.
	 * 
	 * @param productId
	 *            The id of the product to be withdrawn.
	 */
	public void withdrawFromForecast(String productId)
	{
		forecastedProducts.remove(productId);
	}

	/**
	 * Checks whether the specified product's sales are forecasted.
	 * 
	 * @param productId
	 *            The id of the product to check.
	 * @return True if the product's sales are forecasted, false otherwise.
	 */
	public boolean isForecasted(String productId)
	{
		return forecastedProducts.contains(productId);
	}

	/**
	 * Returns a list of products which sales are forecasted.
	 * 
	 * @return A list of products which sales are forecasted.
	 */
	public List<String> getForecastedProducts()
	{
		return new ArrayList<String>(forecastedProducts);
	}

}
