package ibspan.ecap.merchant.shopdecisions.negotiationpreparation;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * A class containing information about a negotiation required to prepare a template and  a strategy. 
 * @author  Micha Drozdowicz
 */
public class NegotiationDescription implements Serializable
{
	private Date preparationTime;
	private String productId;
	private float startingPrice;
	private float reservePrice;
	private int offeredAmount;
	private String strategyId;
	private String templateId;
	
	/**
	 * Constructor
	 *
	 * @param productId the product id being offered
	 * @param startingPrice the starting price to use in the negotiation
	 * @param offeredAmount the amount of product offered
	 */
	public NegotiationDescription(Date preparationTime, String productId, float startingPrice, float reservePrice, int offeredAmount, String strategyId, String templateId)
	{
		this.preparationTime = preparationTime;
		this.productId = productId;
		this.offeredAmount = offeredAmount;
		this.startingPrice = startingPrice;
		this.reservePrice = reservePrice;
		this.strategyId = strategyId;
		this.templateId = templateId;
	}

	/**
	 * Gets the product id being offered.
	 * 
	 * @return the product id being offered.
	 */
	public String getProductId()
	{
		return productId;
	}

	/**
	 * Gets the starting price to use in the negotiation.
	 *
	 * @return the starting price to use in the negotiation.
	 */
	public float getStartingPrice()
	{
		return startingPrice;
	}
	
	/**
	 * Gets the reserve price to use in the negotiation.
	 *
	 * @return the reserve price to use in the negotiation.
	 */
	public float getReservePrice()
	{
		return reservePrice;
	}
	
	/**
	 * Gets the offered amount of product.
	 *
	 * @return the offered amount of product
	 */
	public int getOfferedAmount()
	{
		return offeredAmount;
	}
	
//	private String productId;
//	private float startingPrice;
//	private float reservePrice;
//	private int offeredAmount;
	
	@Override
	public String toString() {
		return String.format("NegotiationDescription: {\n" +
				"\tpreparationTime: %s\n"+
				"\tproductId: %s\n" +
				"\tstartingPrice: %s\n" +
				"\treservePrice: %s\n" +
				"\tofferedAmount: %s\n" +
				"}\n", 
				new SimpleDateFormat().format(preparationTime), productId, startingPrice, reservePrice, offeredAmount);
	}

	public String getStrategyId() {
		return strategyId;
	}

	public String getTemplateId() {
		return templateId;
	}

	public Date getPreparationTime() {
		return preparationTime;
	}	
}
