package ibspan.ecap.merchant.shopdecisions.negotiationpreparation;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.CyclicBehaviour;
import jade.lang.acl.ACLMessage;
import jade.util.Logger;

import java.io.IOException;
import java.util.concurrent.ExecutionException;
import java.util.logging.Level;

/**
 * A behaviour aquiring completed results and dispatching them to the Shop Agent. Should be run  in a separate thread, because while waiting for completed tasks it blocks the running thread.
 * @author  Micha Drozdowicz
 */
public class NegotiationTemplateDispatcher extends CyclicBehaviour
{
	private NegotiationPreparingModule module; 
	private AID shopAgent;
	
	protected Logger myLogger = Logger.getMyLogger(this.getClass().getName()); 
	
	public NegotiationTemplateDispatcher(Agent myAgent, AID shopAgent)
	{
		super(myAgent);
		module = NegotiationPreparingModule.getInstance();
		this.shopAgent = shopAgent;	
	}
	
	@Override
	public void action() {
		myLogger.entering(this.getClass().getName(), "action()");
		try
		{
			myLogger.fine("Taking result");
			NegotiationDescription result = module.takeResult();
			if(result!=null)
			{
				myLogger.fine("Dispatching negotiation description");
				dispatchDescription(result);
			}
		}
		catch(InterruptedException interrupted)
		{
			myLogger.warning("Interrupted while polling updated negotiation description.");					
		}
		catch(ExecutionException ignore){
			if(myLogger.isLoggable(Level.WARNING))
				myLogger.warning("Updating negotiation description failed. Exception: "+ignore.toString());
		}
	}	
	
	private void dispatchDescription(NegotiationDescription template) {

		if(myLogger.isLoggable(Level.INFO))
			myLogger.info("Sending negotiation description to the SA. Template:"+template.toString());
		
		ACLMessage negoParamsMsg = new ACLMessage(ACLMessage.INFORM);
		negoParamsMsg.addReceiver(shopAgent);
		negoParamsMsg.setOntology("NegotiationParameters");				
		try
		{
			negoParamsMsg.setContentObject(template);
		} catch (IOException e)
		{				
			myLogger.severe("Error setting negotiation template params message content.");
			throw new RuntimeException("Error setting negotiation template params message content.",e);
		}            
		myAgent.send(negoParamsMsg);
	}

}
