package ibspan.ecap.merchant.shopdecisions.negotiationpreparation;

import ibspan.ecap.shared.util.TimeConverter;
import jade.core.Agent;
import jade.core.behaviours.SimpleBehaviour;
import jade.util.Logger;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * A behaviour that periodically runs negotiation template tasks. It runs tasks for different products one after another, each in its own action method call, waiting a certain time  (specified in the constructor) between consective calls. After the behaviour goes through all the registered products it starts from the beginning.
 * @author  Micha Drozdowicz
 */
public class NegotiationTemplateUpdater extends SimpleBehaviour
{
	private NegotiationPreparingModule module;
	private long secondsBetweenUpdates;
	private long secondsBetweenCycles;
	private Date lastWakeUp;
	private boolean repeat;
	private boolean finished = false;
	private boolean isUpdating = false;
	
	protected Logger myLogger = Logger.getMyLogger(this.getClass().getName());
	
	/**
	 * Constructor
	 *
	 * @param myAgent The agent this behaviour must belong to.
	 * @param secondsBetweenUpdates The amount of time to block the behaviour for between 
	 * consective product template update calls. 
	 */
	public NegotiationTemplateUpdater(Agent myAgent, long secondsBetweenUpdates, long secondsBetweenCycles, boolean repeat)
	{
		super(myAgent);
		module = NegotiationPreparingModule.getInstance();
		this.secondsBetweenUpdates = secondsBetweenUpdates;
		this.secondsBetweenCycles = secondsBetweenCycles;
		lastWakeUp = new Date(0); 	//tak dawno, ze nigdy ;)
		this.repeat = repeat;
	}

	@Override
	public void action()
	{
		TimeConverter tc = TimeConverter.getInstance();
		Date now = tc.getDate();
		long secondsSinceLastWakeup = (now.getTime() - lastWakeUp.getTime())/1000;
		
		myLogger.finer("Negotiation template updater woke up. Now: "+new SimpleDateFormat().format(now)
				+"; last wake up: "+new SimpleDateFormat().format(lastWakeUp)+"; seconds since wake up: "+secondsSinceLastWakeup);
		
		

		if ((isUpdating && secondsSinceLastWakeup > secondsBetweenUpdates)
				|| (!isUpdating && secondsSinceLastWakeup > secondsBetweenCycles))
		{
			lastWakeUp = now;
			isUpdating = true;
			module.requestNextNegotiationDescription();
			if (module.roundFinished())
			{
				isUpdating = false;
				if (!repeat)
				{					
					finished = true;
				}
				else
				{
					myLogger.finer("Blocking for cycle "+secondsBetweenCycles);
					block(tc.convertSpanFromReal(1000*secondsBetweenCycles));
				}
			} else
			{
				myLogger.finer("Blocking for update "+secondsBetweenUpdates);
				block(tc.convertSpanFromReal(1000*secondsBetweenUpdates));
			}
		} else
		{
			if(isUpdating)
			{
				myLogger.finer("Blocking for less then update "+(secondsBetweenUpdates
						- secondsSinceLastWakeup));
				block(tc.convertSpanFromReal(1000*(secondsBetweenUpdates
						- secondsSinceLastWakeup)));
			}
			else
			{
				myLogger.finer("Blocking for less then cycle "+(secondsBetweenCycles
						- secondsSinceLastWakeup)+"; compressed: "+tc.convertSpanFromReal(secondsBetweenCycles
								- secondsSinceLastWakeup));
				block(tc.convertSpanFromReal(1000*(secondsBetweenCycles
						- secondsSinceLastWakeup)));
			}
		}
			
	}

	@Override
	public boolean done()
	{
		return finished;
	}

}
