package ibspan.ecap.merchant.shopdecisions.negotiationpreparation;

import java.util.Date;
import java.util.concurrent.Callable;
import java.util.concurrent.CompletionService;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executor;
import java.util.concurrent.ExecutorCompletionService;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * A class responsible for running {@link PreparationTask}s and serving as a completion service
 * providing methods to retrieve completed task results.
 *
 * @author Micha Drozdowicz
 *
 */
public class PreparationTaskRunner
{
	private CompletionService<NegotiationDescription> service;

	private int coreThreadPoolSize = 2;

	private int maxThreadPoolSize = 10;

	private long threadKeepAliveTime = 120L; // in seconds
	
	public PreparationTaskRunner()
	{
		Executor executor = new ThreadPoolExecutor(coreThreadPoolSize, maxThreadPoolSize, threadKeepAliveTime, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>());
		service = new ExecutorCompletionService<NegotiationDescription>(executor);		
	}
	
	/**
	 * Performs a blocking receive of a finished task. Blocks the execution of the calling thread
	 * until a task completes and the result is available.
	 *
	 * @return the result of a completed task.
	 * @throws InterruptedException when the calling thread has been interrupted while waiting
	 * for the result
	 */
	public NegotiationDescription takeResult() throws InterruptedException, ExecutionException
	{
		NegotiationDescription result = null;
		Future<NegotiationDescription> f = service.take();
		if(f!=null)
		{
			result = f.get();
		}
	
		return result;
	}

	/**
	 * Executes the specified task.  
	 *
	 * @param task the task to be executed
	 * @return the {@link Future} describing the future result of the task.
	 */
	public Future<NegotiationDescription> runPreparationTask(PreparationTask task)	
	{
		return service.submit(task);				
	}


}
