package ibspan.ecap.merchant.stubs;

import ibspan.ecap.merchant.shopdecisions.dataadministration.DemandRequest;
import ibspan.ecap.merchant.shopdecisions.dataadministration.FinalisationType;
import ibspan.ecap.merchant.shopdecisions.dataadministration.TransactionLog;
import ibspan.ecap.merchant.shopdecisions.negotiationpreparation.NegotiationDescription;
import ibspan.ecap.shared.ontology.negotiation.BidInfo;
import ibspan.ecap.shared.ontology.negotiation.DemandData;
import ibspan.ecap.shared.ontology.negotiation.NegotiationLog;
import ibspan.ecap.shared.ontology.negotiation.ReservationInfo;
import ibspan.ecap.shared.util.MyMath;
import ibspan.ecap.shared.util.PropertiesProvider;
import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.FSMBehaviour;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.UnreadableException;
import jade.util.Logger;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Random;
import java.util.UUID;
import java.util.logging.Level;

/**
 * An agent mimicking the behaviour of the ShopAgent in interactions with the ShopDecisionAgent.
 * 
 * @author Michal Drozdowicz
 *
 */
public class ShopAgentStub extends Agent
{ 
	public class InitiatingSimulation extends OneShotBehaviour
	{
		public InitiatingSimulation(Agent a)
		{
			super(a);
		}

		@Override
		public void action()
		{
			String productId = PropertiesProvider.getInstance().getStringProperty(PRODUCT_ID_PROP_NAME, DEF_PRODUCT_ID);
			myLogger.info("Shop Agent sending product introduction message. ProductId = " + productId);
															
			ACLMessage introMsg = new ACLMessage(ACLMessage.INFORM);
			introMsg.addReceiver(sda);
			introMsg.setOntology("productIntroduced");				
			try
			{
				introMsg.setContentObject(productId);
			} catch (IOException e)
			{				
				myLogger.log(Level.WARNING, "Error setting demand data message content.", e);
				throw new RuntimeException("Error setting demand data message content.",e);
			}            
			myAgent.send(introMsg);
			
			myLogger.info("Shop Agent sending product start forecasting message. ProductId = " + productId);
			
//			ACLMessage startForecastMsg = new ACLMessage(ACLMessage.INFORM);
//			startForecastMsg.addReceiver(sda);
//			startForecastMsg.setOntology("startForecasting");				
//			try
//			{
//				startForecastMsg.setContentObject(productId);
//			} catch (IOException e)
//			{				
//				myLogger.log(Level.WARNING, "Error setting demand data message content.", e);
//				throw new RuntimeException("Error setting demand data message content.",e);
//			}            
//			myAgent.send(startForecastMsg);
		}

	}

	//behaviours
	public class MainBehaviour extends FSMBehaviour
	{
		private static final String PROVIDING_DEMAND = "providingDemand";
		private static final String PROVIDING_NEGOTIATIONS = "providingNegotiations";
		public static final String CALCULATING_SALES = "calculatingSales";
		public static final String SYNCHRONISING_TIME = "synchronisingTime";
		private static final String INITIATING = "initiating";
		private static final String FINALISING_NEGOTIATIONS = "finalisingNegotiations";

		private static final int DONE = 0;
		private static final int NOT_DONE = 1;
		private static final int MORE_NEGOS = 1;
		private static final int NO_NEGOS = 0;
				
		public MainBehaviour(Agent a)
		{
			
			super(a);
			
			//registerFirstState(new SynchronisingTime(a), SYNCHRONISING_TIME);
			registerFirstState(new InitiatingSimulation(a), INITIATING);
			registerState(new ProvidingDemandData(a), PROVIDING_DEMAND);
			registerState(new CalculatingSales(a), CALCULATING_SALES);
			registerState(new ProvidingNegotiationData(a), PROVIDING_NEGOTIATIONS);
			registerState(new FinalisingReservations(a), FINALISING_NEGOTIATIONS);
			
			registerTransition(INITIATING, PROVIDING_DEMAND, DONE);
			//registerTransition(SYNCHRONISING_TIME, PROVIDING_DEMAND, OK);
			registerTransition(PROVIDING_DEMAND, PROVIDING_DEMAND, NOT_DONE);
			registerTransition(PROVIDING_DEMAND, CALCULATING_SALES, DONE);
			registerTransition(CALCULATING_SALES, CALCULATING_SALES, NOT_DONE);
			registerTransition(CALCULATING_SALES, PROVIDING_NEGOTIATIONS, DONE);
			registerTransition(PROVIDING_NEGOTIATIONS, PROVIDING_NEGOTIATIONS, MORE_NEGOS);
			//registerTransition(PROVIDING_NEGOTIATION, SYNCHRONISING_TIME, NO_NEGOS);
			registerTransition(PROVIDING_NEGOTIATIONS, FINALISING_NEGOTIATIONS, NO_NEGOS);
			registerTransition(FINALISING_NEGOTIATIONS, FINALISING_NEGOTIATIONS, MORE_NEGOS);
			registerTransition(FINALISING_NEGOTIATIONS, PROVIDING_DEMAND, NO_NEGOS);			
		}
		
		public class ProvidingDemandData extends OneShotBehaviour
		{		
			private int status = NOT_DONE; 
			public ProvidingDemandData(Agent a)
			{
				super(a);			
				
			}

			@Override
			public void action()
			{
				status = NOT_DONE;
				myLogger.info("Shop Agent stub waiting for demand requests.");
				MessageTemplate mt = MessageTemplate.MatchSender(sda);
				mt = MessageTemplate.and(mt, MessageTemplate.MatchOntology("DemandRequest"));
				ACLMessage demandReqMsg = myAgent.receive(mt);
				if(demandReqMsg!=null)
				{					
					status = DONE;
					DemandRequest req;
					try
					{
						req = (DemandRequest) demandReqMsg.getContentObject();
					}
					catch(ClassCastException e)
					{
						myLogger.log(Level.SEVERE, "Message contained wrong content object.", e);
						throw new RuntimeException("Message contained wrong content object.", e);
					}
					catch (UnreadableException e)
					{
						myLogger.log(Level.SEVERE, "Could not read message contents.", e);
						throw new RuntimeException("Could not read message contents.", e);
					}
					myLogger.info("Shop Agent stub received demand requests. Going to parse demand file.");
					
					Date yesterday = req.getDate();
					Calendar cal = new GregorianCalendar();
					cal.setTime(yesterday);
					cal.add(Calendar.DATE, 1);
					cal.set(Calendar.HOUR, 0);
					cal.set(Calendar.MINUTE, 0);
					cal.set(Calendar.SECOND, 0);
					currentDate = cal.getTime();
					
					previousDemand = currentDemand;
					try
					{
						currentDemand = parseNextDemand();
					} catch (IOException e1)
					{
						myLogger.log(Level.SEVERE, "Failed to parse demand file.", e1);
						throw new RuntimeException("Failed to parse demand file.");
					}
					
					try
					{
						currentPrice = parseNextPrice();
					}catch (IOException e1)
					{
						myLogger.log(Level.SEVERE, "Failed to parse price file.", e1);
						throw new RuntimeException("Failed to parse price file.");
					}
					
					myLogger.info("Shop Agent retrieved demand info. Going to send demand data.");
					
					DemandData demand;
					demand = previousDemand;
					demand.setGlobalProductId(req.getProductId());
					demand.setWhen(req.getDate());					
															
					ACLMessage demandMsg = new ACLMessage(ACLMessage.INFORM);
					demandMsg.addReceiver(sda);
					demandMsg.setOntology("DemandData");				
					try
					{
						demandMsg.setContentObject(demand);
					} catch (IOException e)
					{				
						myLogger.log(Level.SEVERE, "Error setting demand data message content.", e);
						throw new RuntimeException("Error setting demand data message content.",e);
					}            
					myAgent.send(demandMsg);	
					myLogger.info("Shop Agent stub sent demand data.");
				}
				else
				{
					block();
				}								
			}
			
			@Override
			public int onEnd()
			{
				return status;
			}		
		}
		
		public class CalculatingSales extends OneShotBehaviour
		{				
			private int status = NOT_DONE;

			public CalculatingSales(Agent a)
			{
				super(a);
			}

			@Override
			public void action()
			{
				status = NOT_DONE;
				myLogger.info("Shop Agent stub waiting for negotiation parameters.");
				MessageTemplate mt = MessageTemplate.MatchSender(sda);
				mt = MessageTemplate.and(mt, MessageTemplate.MatchOntology("NegotiationParameters"));				
				ACLMessage tempMsg;
				NegotiationDescription descr = null;
				//Read all pending messages and choose the latest one. Discard older than today's.
				while ((tempMsg = myAgent.receive(mt)) != null) {
					NegotiationDescription tempDescr;
					try
					{						
						tempDescr = (NegotiationDescription) tempMsg.getContentObject();
						if(myLogger.isLoggable(Level.FINER))
						{
							myLogger.log(Level.FINER, "Received negotiation params : "+descr);
						}
					}
					catch(ClassCastException e)
					{
						myLogger.log(Level.SEVERE, "Message contained wrong content object.", e);
						throw new RuntimeException("Message contained wrong content object.", e);
					}
					catch (UnreadableException e)
					{
						myLogger.log(Level.SEVERE, "Could not read message contents.", e);
						throw new RuntimeException("Could not read message contents.", e);
					}
					if(!tempDescr.getPreparationTime().before(currentDate) &&(descr == null || tempDescr.getPreparationTime().after(descr.getPreparationTime())))
					{
						descr = tempDescr;
					}					
				}
				if(descr!=null)
				{
					status = DONE;					
					
					myLogger.info("Shop Agent stub received negotiation parameters. Calculating sales amount.");
					
					negotiations = createNegotiationLogs(descr);					
				}
				else
				{
					block();
				}

			}
			
			@Override
			public int onEnd()
			{
				return status;
			}

			private ArrayList<NegotiationLog> createNegotiationLogs(NegotiationDescription descr)
			{
				myLogger.log(Level.INFO, "Creating negotiation logs.Date: "+currentDate+" Negotiation parameters:"+descr.toString());
				//simulation parameters
				PropertiesProvider propsProvider = PropertiesProvider.getInstance();
				int negotiationCountMean = propsProvider.getIntProperty(NEGOTIATION_COUNT_MEAN_PROP_NAME, DEF_NEGOTIATION_COUNT_MEAN);
				int negotiationCountVariance = propsProvider.getIntProperty(NEGOTIATION_COUNT_VARIANCE_PROP_NAME, DEF_NEGOTIATION_COUNT_VARIANCE);
				long firstNegoDelay = propsProvider.getLongProperty(FIRST_NEGO_DELAY_PROP_NAME, DEF_FIRST_NEGO_DELAY);
				double negoSoldVarianceRatio = propsProvider.getDoubleProperty(NEGO_SOLD_VARIANCE_RATIO_PROP_NAME, DEF_NEGO_SOLD_VARIANCE_RATIO);
				long negotiationLength = propsProvider.getLongProperty(NEGOTIATION_LENGTH_PROP_NAME, DEF_NEGOTIATION_LENGTH);
				long offerDelay = propsProvider.getLongProperty(OFFER_DELAY_PROP_NAME, DEF_OFFER_DELAY);
				long reservationExpiryLength = propsProvider.getLongProperty(RESERVATION_EXPIRY_LENGTH_PROP_NAME, DEF_RESERVATION_EXPIRY_LENGTH);
				double clientPriceVarianceRatio = propsProvider.getDoubleProperty(CLIENT_PRICE_VARIANCE_RATIO_PROP_NAME, DEF_CLIENT_PRICE_VARIANCE_RATIO);
				
				int salesCount = calculateSalesCount(descr.getStartingPrice(), clientPriceVarianceRatio);
				if(salesCount == 0)
				{
					myLogger.fine("Didn't manage to sell any products. Description:"+descr.toString());
				}
				ArrayList<NegotiationLog> list = new ArrayList<NegotiationLog>(salesCount);
				if (salesCount > 0) {
					long millisBetweenNegos = (1000 * 60 * 60 * 22)
							/ salesCount;

					Date firstNegoTime = new Date(currentDate.getTime()
							+ firstNegoDelay);

					Random rand = new Random(new Date().getTime());

					for (int i = 0; i < salesCount; i++) {
						Date negoStartTime = new Date(firstNegoTime.getTime()
								+ millisBetweenNegos * i);
						int negoSoldMean = currentDemand
								.getMeanAmountPerClient();
						NegotiationLog log = new NegotiationLog();
						int negoSold = (int) MyMath.gaussian(negoSoldMean,
								negoSoldVarianceRatio * negoSoldMean);
						log.setAskingPrice(descr.getStartingPrice());
						log.setReservePrice(descr.getReservePrice());
						log.setNegotiationId(Math.abs((int) UUID.randomUUID()
								.getLeastSignificantBits()));
						log.setProductId(descr.getProductId());
						log.setQuantityOffered(descr.getOfferedAmount());
						log.setStrategyId(descr.getStrategyId());
						log.setTemplateId(descr.getTemplateId());
						log.setNegotiationStart(negoStartTime);
						log.setNegotiationEnd(new Date(negoStartTime.getTime()
								+ negotiationLength));

						List<BidInfo> bids = new ArrayList<BidInfo>(1);
						BidInfo bid = new BidInfo();
						bid.setOfferTime(new Date(negoStartTime.getTime()
								+ offerDelay));
						bid.setPrice(descr.getReservePrice());
						bid.setQuantity(Math.min(negoSold, descr
								.getOfferedAmount()));
						bid.setWinning(true);
						bid.setClientId("Client"
								+ rand.nextInt(currentDemand
										.getRegisteredClients()));
						ReservationInfo res = new ReservationInfo();
						res.setExpiryTime(new Date(bid.getOfferTime().getTime()
								+ reservationExpiryLength));
						res.setTransactionId(UUID.randomUUID().toString());
						bid.setReservation(res);
						bids.add(bid);
						log.setBids(bids);

						list.add(log);
					}
				}
					
				return list;				
			}

			private int calculateSalesCount(float shopPrice, double clientPriceVarianceRatio)
			{
				int clients = currentDemand.getRegisteredClients();
				
				double clientMeanPrice = calculateClientMeanPrice();
							
				
				int soldAmount = (int) (clients * (1-MyMath.Phi((double)shopPrice, (double) clientMeanPrice, clientPriceVarianceRatio * clientMeanPrice)));
				myLogger.fine("Calculated sales count. Clients: "+clients+"; \nClient mean price: "
						+clientMeanPrice+"\n Client price variation: "+clientPriceVarianceRatio*clientMeanPrice
						+"\nSold amount: "+soldAmount);
				return soldAmount;								
			}

			private double calculateClientMeanPrice()
			{
				return currentPrice; 
			}		
		}
		
		public class ProvidingNegotiationData extends OneShotBehaviour
		{
			private int transition; 
			
			public ProvidingNegotiationData(Agent a)
			{
				super(a);
			}

			@Override
			public void action()
			{
//				NegotiationLog log = getNextNegotiationLog();
//				if(log != null)
//				{
//					myLogger.info("Shop Agent stub sending negotiation log.");
//					ACLMessage logMsg = new ACLMessage(ACLMessage.INFORM);
//					logMsg.addReceiver(sda);
//					logMsg.setOntology("NegotiationLog");
//					try
//					{
//						logMsg.setContentObject(log);
//					} catch (IOException e)
//					{				
//						myLogger.log(Level.SEVERE, "Error setting negotiation log message content.", e);
//						throw new RuntimeException("Error setting negotiation log message content.",e);
//					}            
//					myAgent.send(logMsg);	
//					transition = MORE_NEGOS;
//				}
//				else
//				{
//					myLogger.info("Finished sending negotiation logs.");
//					transition = NO_NEGOS;
//					currentNegotiationIndex = 0;
//				}
				
				//^ disabled for testing performance
				myLogger.info("Shop Agent stub sending negotiation log.");
				ACLMessage logMsg = new ACLMessage(ACLMessage.INFORM);
				logMsg.addReceiver(sda);
				logMsg.setOntology("negotiationLog");
				try
				{
					logMsg.setContentObject(negotiations.toArray(new NegotiationLog[0]));
				} catch (IOException e)
				{				
					myLogger.log(Level.SEVERE, "Error setting negotiation log message content.", e);
					throw new RuntimeException("Error setting negotiation log message content.",e);
				}            
				myAgent.send(logMsg);	
				transition = NO_NEGOS;
			}
			
			@Override
			public int onEnd()
			{
				return transition;
			}
		}
		
		public class FinalisingReservations extends OneShotBehaviour {

			private int transition; 
			
			public FinalisingReservations(Agent a)
			{
				super(a);
			}

			@Override
			public void action()
			{
//				NegotiationLog log = getNextNegotiationLog();
//				if(log != null)
//				{
//					myLogger.info("Shop Agent stub finalising negotiation.");
//					ACLMessage logMsg = new ACLMessage(ACLMessage.INFORM);
//					logMsg.addReceiver(sda);
//					logMsg.setOntology("TransactionLog");
//					for (BidInfo bi : log.getBids()) {
//						if (bi.isWinning()) {
//							try {
//								TransactionLog transLog = new TransactionLog();
//								transLog.setClientId(bi.getClientId());
//								transLog
//										.setFinalisationType(FinalisationType.CONFIRMED);
//								transLog.setNegotiationId(log
//										.getNegotiationId());
//								transLog.setQuantityReserved(bi.getQuantity());
//								transLog.setTime(log.getNegotiationEnd());
//								transLog.setTransactionId(bi.getReservation()
//										.getTransactionId());
//								transLog.setValueReserved(bi.getPrice()
//										* bi.getQuantity());
//								logMsg.setContentObject(transLog);
//							} catch (IOException e) {
//								myLogger.log(Level.SEVERE, "Error setting reservation log message content.", e);
//								throw new RuntimeException(
//										"Error setting reservation log message content.",
//										e);
//							}
//						}
//					}
//					myAgent.send(logMsg);	
//					transition = MORE_NEGOS;
//				}
//				else
//				{
//					myLogger.info("Finished sending negotiation logs.");
//					transition = NO_NEGOS;
//					currentNegotiationIndex = 0;
//				}
				
				ArrayList<TransactionLog> tlogs = new ArrayList<TransactionLog>();
				for (NegotiationLog log : negotiations) {
					for (BidInfo bi : log.getBids()) {
						if (bi.isWinning()) {

							TransactionLog transLog = new TransactionLog();
							transLog.setClientId(bi.getClientId());
							transLog
									.setFinalisationType(FinalisationType.CONFIRMED);
							transLog.setNegotiationId(log.getNegotiationId());
							transLog.setQuantityReserved(bi.getQuantity());
							transLog.setTime(log.getNegotiationEnd());
							transLog.setTransactionId(bi.getReservation()
									.getTransactionId());
							transLog.setValueReserved(bi.getPrice()
									* bi.getQuantity());
							tlogs.add(transLog);

						}
					}
				}
				ACLMessage logMsg = new ACLMessage(ACLMessage.INFORM);
				logMsg.addReceiver(sda);
				logMsg.setOntology("transactionLog");
				try {
					logMsg.setContentObject(tlogs
							.toArray(new TransactionLog[0]));
				} catch (IOException e) {
					myLogger
							.log(
									Level.SEVERE,
									"Error setting reservation log message content.",
									e);
					throw new RuntimeException(
							"Error setting reservation log message content.", e);
				}
				myAgent.send(logMsg);

				myLogger.info("Sent transaction logs. Size: " + tlogs.size());
				transition = NO_NEGOS;
				currentNegotiationIndex = 0;
			}				
			
			
			@Override
			public int onEnd()
			{
				return transition;
			}
		}
		
//		public class SynchronisingTime extends OneShotBehaviour
//		{
//
//			public SynchronisingTime(Agent a)
//			{
//				super(a);
//			}
//
//			@Override
//			public void action()
//			{
//				advanceDate();
//				previousDemand = currentDemand;
//				try
//				{
//					currentDemand = parseNextDemand();
//				} catch (IOException e1)
//				{
//					e1.printStackTrace();
//					throw new RuntimeException("Failed to parse demand file.");
//				}
//				
//				ACLMessage synchroMsg = new ACLMessage(ACLMessage.INFORM);
//				synchroMsg.addReceiver(sda);
//				synchroMsg.setOntology("Synchronisation");
//				try
//				{
//					synchroMsg.setContentObject(currentDate);
//				} catch (IOException e)
//				{				
//					e.printStackTrace();
//					throw new RuntimeException("Error setting negotiation log message content.",e);
//				}            
//				myAgent.send(synchroMsg);	
//			}
//
//			@Override
//			public int onEnd()
//			{
//				return DONE;
//			}
//		}

	}
		
	private BufferedReader demandReader;
	private BufferedReader priceReader;
	private DemandData currentDemand;
	private DemandData previousDemand;
	private double currentPrice;
	private ArrayList<NegotiationLog> negotiations;
	private AID sda;
	private int currentNegotiationIndex;
	private Date currentDate;
	
	private static final String DEF_PROP_FILE_NAME = "sastub.properties";
	private static final String DEF_DEMAND_FILE_NAME = "demand.txt";
	private static final String DEF_PRICE_FILE_NAME = "price.txt";
	private static final String DEF_SDA_NAME = "sda";
	private static final String DEF_PRODUCT_ID = "Blender ZX-2012-TTR";
	private static final int DEF_NEGOTIATION_COUNT_MEAN = 10;
	private static final int DEF_NEGOTIATION_COUNT_VARIANCE = 2;
	private static final long DEF_FIRST_NEGO_DELAY = 1000*60*10;
	private static final long DEF_NEGOTIATION_LENGTH = 1000*5;
	private static final double DEF_NEGO_SOLD_VARIANCE_RATIO = 0.1;
	private static final long DEF_OFFER_DELAY = 1000*5;
	private static final long DEF_RESERVATION_EXPIRY_LENGTH = 1000*60*60*2;
	private static final double DEF_CLIENT_PRICE_VARIANCE_RATIO = 0.2;
			
	private static final String DEMAND_FILE_NAME_PROP_NAME = "demandFileName";
	private static final String PRICE_FILE_NAME_PROP_NAME = "priceFileName";
	private static final String SDA_PROP_NAME = "sdaName";
	private static final String PRODUCT_ID_PROP_NAME = "productId";
	private static final String NEGOTIATION_COUNT_MEAN_PROP_NAME = "negotiationCountMean";
	private static final String NEGOTIATION_COUNT_VARIANCE_PROP_NAME = "negotiationCountVariance";
	private static final String FIRST_NEGO_DELAY_PROP_NAME = "firstNegoDelay";
	private static final String NEGOTIATION_LENGTH_PROP_NAME = "negotiationLength";
	private static final String NEGO_SOLD_VARIANCE_RATIO_PROP_NAME = "negoSoldVarianceRatio";
	private static final String OFFER_DELAY_PROP_NAME = "offerDelay";
	private static final String CLIENT_PRICE_VARIANCE_RATIO_PROP_NAME = "bidSoldVarianceRatio";
	private static final String RESERVATION_EXPIRY_LENGTH_PROP_NAME = "reservationExpiryLength";
			
	private Logger myLogger = Logger.getMyLogger(this.getClass().getName());;
		
	@Override
	protected void setup()
	{		
		super.setup();
		
		Object[] args = getArguments();
		String propFileName = "";
		for (int i = 0; i < args.length; i++) {
			propFileName += args[i]+" ";
		}
		propFileName = propFileName.trim();
		if(propFileName.length() == 0)
			propFileName = DEF_PROP_FILE_NAME;
//		String propFileName = (args!=null && args.length > 0) ? args[0].toString() : DEF_PROP_FILE_NAME;

		myLogger.fine("Property file name: "+propFileName);
		
		PropertiesProvider propertyProvider = PropertiesProvider.getInstance();
		propertyProvider.loadProperties(propFileName, args);
		
		
		myLogger.info("Loaded properties:\n"+propertyProvider.getPropertyList());
		
		String sdaName = propertyProvider.getStringProperty(SDA_PROP_NAME, DEF_SDA_NAME);
		sda = new AID(sdaName, AID.ISLOCALNAME);
		
		String demandFilePath = propertyProvider.getStringProperty(DEMAND_FILE_NAME_PROP_NAME, DEF_DEMAND_FILE_NAME);
		myLogger.fine("Demand file name: "+demandFilePath);
		try
		{
			demandReader = new BufferedReader(new FileReader(demandFilePath));
		} catch (FileNotFoundException e)
		{
			myLogger.log(Level.SEVERE, "Demand file not found.", e);
			throw new RuntimeException("Demand file not found");
		}
		
		String priceFilePath = propertyProvider.getStringProperty(PRICE_FILE_NAME_PROP_NAME, DEF_PRICE_FILE_NAME);
		try
		{
			priceReader = new BufferedReader(new FileReader(priceFilePath));
		} catch (FileNotFoundException e)
		{
			myLogger.log(Level.SEVERE, "Price file not found.", e);
			throw new RuntimeException("Price file not found");
		}
		
		addBehaviour(new MainBehaviour(this));
		//currentDate = new GregorianCalendar(2007, 1, 1).getTime(); //TODO: Skd data?
		try
		{
			currentDemand = parseNextDemand();
			currentPrice = parseNextPrice();
		} catch (IOException e)
		{
			myLogger.log(Level.WARNING, "Could not parse demand file.", e);
			throw new RuntimeException("Could not parse demand file.");
		}
	}
	
	@Override
	protected void takeDown()
	{		
		super.takeDown();
		try
		{
			demandReader.close();
		} catch (IOException e)
		{
			//ignore
			myLogger.log(Logger.FINER, "Error occurred while closing demand file reader.", e);
		}
		try
		{
			priceReader.close();
		} catch (IOException e)
		{
			//ignore
			myLogger.log(Logger.FINER, "Error occurred while closing price file reader.", e);
		}
	}
	
	private DemandData parseNextDemand() throws IOException
	{	
		DemandData data = new DemandData();
		
		String line = demandReader.readLine();
		String regex = "\t";		
		String[] params = line.split(regex);
		//System.out.printf("Line: %1$s", line);
		//System.out.printf("Demand: %1$s; Shops: %2$s; Clients: %3$s\n", params[0], params[1], params[2]);
		int cicQueries = Integer.parseInt(params[0]);
		int shops = Integer.parseInt(params[1]);
		int clients = Integer.parseInt(params[2]);
		int meanAmountPerClient = Integer.parseInt(params[3]);
		
		data.setCicQueryCount(cicQueries);
		data.setShopsSellingProductCount(shops);		
		
		data.setRegisteredClients(clients);
		data.setMeanAmountPerClient(meanAmountPerClient);
		
		return data;
	}	
	
	private double parseNextPrice() throws IOException
	{			
		String line = priceReader.readLine();
		
		return Double.parseDouble(line);
	}	
	
//	private void advanceDate()
//	{
//		Calendar cal = new GregorianCalendar();
//		cal.setTime(currentDate);
//		cal.add(Calendar.DATE, 1);
//		currentDate.setTime(cal.getTimeInMillis());
//	}
	
	private NegotiationLog getNextNegotiationLog()
	{
		if(negotiations!=null && currentNegotiationIndex < negotiations.size())
			return negotiations.get(currentNegotiationIndex++);
		else
			return null;
	}
}
