/*****************************************************************
 E-commerce Agent-based Platform
 Copyright (C) 2006-2008 SRI PAS

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor,  Boston, MA  
 02110-1301, USA.
 *****************************************************************/
package ibspan.ecap.shared.ontology.negotiation;

import jade.content.Concept;

import java.util.Date;

/**
 * 
 * @author Micha Drozdowicz
 */
public class BidInfo implements Concept
{	
	private String clientId;
	private Date offerTime;
	private int quantity;
	private float price;
	private boolean isWinning;
	private ReservationInfo reservation;
	
	public BidInfo(String clientId, Date offerTime, int quantity, float price,
			boolean isWinning, ReservationInfo reservation) {
		super();
		this.clientId = clientId;
		this.offerTime = offerTime;
		this.quantity = quantity;
		this.price = price;
		this.isWinning = isWinning;
		this.reservation = reservation;
	}
	public BidInfo() {
		// TODO Auto-generated constructor stub
	}
	/**
	 * Gets the time the offer was made.
	 *
	 * @return the time the offer was made
	 */
	public Date getOfferTime()
	{
		return offerTime;
	}
	/**
	 * Sets the time the offer was made
	 *
	 * @param offerTime the time the offer was made to set
	 */
	public void setOfferTime(Date offerTime)
	{
		this.offerTime = offerTime;
	}
	/**
	 * Gets the quantity offered.
	 *
	 * @return the quantity offered
	 */
	public int getQuantity()
	{
		return quantity;
	}
	/**
	 * Sets the quantity offered.
	 *
	 * @param quantity quantity offered to set
	 */
	public void setQuantity(int quantity)
	{
		this.quantity = quantity;
	}
	/**
	 * Gets the price offered. 
	 *
	 * @return the price offered
	 */
	public float getPrice()
	{
		return price;
	}
	/**
	 * Sets the value of price offered.
	 *
	 * @param price the price to set
	 */
	public void setPrice(float price)
	{
		this.price = price;
	}
	/**
	 * Gets true if the bid was a winning bid and false otherwise. 
	 *
	 * @return true if the bid was a winning bid and false otherwise.
	 */
	public boolean isWinning()
	{
		return isWinning;
	}
	/**
	 * Sets whether the bid was a winning bid or not.
	 *
	 * @param isWinning a boolean value of whether the bid was a winning bid or not.
	 */
	public void setWinning(boolean isWinning)
	{
		this.isWinning = isWinning;
	}
	/**
	 * Gets the information about the reservation following the bid. If the bid wasn't a winning bid, null is returned.
	 *
	 * @return the information about the reservation following the bid or null if the bid wasn't a winning bid
	 */
	public ReservationInfo getReservation()
	{
		return reservation;
	}
	/**
	 * Sets the information about the reservation following the bid. If the bid wasn't a winning bid, null should be passed as an argument.
	 *
	 * @param reservation the information about the reservation following the bid or null if the bid wasn't a winning bid
	 */
	public void setReservation(ReservationInfo reservation)
	{
		this.reservation = reservation;
	}
	/**
	 * Gets the id of the client who made the bid. 
	 * 
	 * @return the id of the client who made the bid
	 */
	public String getClientId() {
		return clientId;
	}
	
	/**
	 * Sets the id of the client who made the bid. 
	 * 
	 * @param clientId the id of the client who made the bid
	 */
	public void setClientId(String clientId) {
		this.clientId = clientId;
	}
	
	@Override
	public String toString() {
		return String.format("BidInfo: {\n" +
				"\tclientId: %s\n" +
				"\tofferTime: %s\n" +
				"\tquantity: %s\n" +
				"\tprice: %s\n" +
				"\tisWinning: %s\n" +
				"\treservation: %s\n" +
				"}\n", 
				clientId, offerTime, quantity, price,
				isWinning, reservation);
	}
}
