/*****************************************************************
 E-commerce Agent-based Platform
 Copyright (C) 2006-2008 SRI PAS

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor,  Boston, MA  
 02110-1301, USA.
 *****************************************************************/
package ibspan.ecap.shared.ontology.negotiation;

import jade.content.Concept;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * A class containing information about a finished negotiation.
 * 
 * @author Micha Drozdowicz
 * 
 */
public class NegotiationLog implements Concept {
	private int negotiationId;
	private String productId;
	private Date negotiationStart;
	private Date negotiationEnd;
	private int quantityOffered;
	private String strategyId;
	private String templateId;
	private float reservePrice;
	private float askingPrice;
	private List<BidInfo> bids;

	/**
	 * Gets the id of the negotiation.
	 * 
	 * @return the id of the negotiation.
	 */
	public int getNegotiationId() {
		return negotiationId;
	}

	/**
	 * Sets the id of the negotiation.
	 * 
	 * @param negotiationId
	 *            the id of the negotiation.
	 */
	public void setNegotiationId(int negotiationId) {
		this.negotiationId = negotiationId;
	}

	/**
	 * Gets the global id of the product.
	 * 
	 * @return the global id of the product.
	 */
	public String getProductId() {
		return productId;
	}

	/**
	 * Sets the global id of the product.
	 * 
	 * @param productId
	 *            the global id of the product.
	 */
	public void setProductId(String productId) {
		this.productId = productId;
	}

	/**
	 * Gets the time of the beginning of the negotiation.
	 * 
	 * @return the time of the beginning of the negotiation
	 */
	public Date getNegotiationStart() {
		return negotiationStart;
	}

	/**
	 * Sets the time of the beginning of the negotiation.
	 * 
	 * @param negotiationStart
	 *            the time of the beginning of the negotiation.
	 */
	public void setNegotiationStart(Date negotiationStart) {
		this.negotiationStart = negotiationStart;
	}

	/**
	 * Gets the time of the end of the negotiation.
	 * 
	 * @return the time of the end of the negotiation
	 */
	public Date getNegotiationEnd() {
		return negotiationEnd;
	}

	/**
	 * Sets the time of the end of the negotiation.
	 * 
	 * @param negotiationEnd
	 *            the time of the end of the negotiation.
	 */
	public void setNegotiationEnd(Date negotiationEnd) {
		this.negotiationEnd = negotiationEnd;
	}

	/**
	 * Gets the quantity offered.
	 * 
	 * @return the quantity offered
	 */
	public int getQuantityOffered() {
		return quantityOffered;
	}

	/**
	 * Sets the quantity offered.
	 * 
	 * @param quantityOffered
	 *            the quantity offered to set
	 */
	public void setQuantityOffered(int quantityOffered) {
		this.quantityOffered = quantityOffered;
	}

	/**
	 * Gets the id of the strategy used.
	 * 
	 * @return the id of the strategy used.
	 */
	public String getStrategyId() {
		return strategyId;
	}

	/**
	 * Sets the id of the strategy used.
	 * 
	 * @param strategyId
	 *            the id of the strategy used to set
	 */
	public void setStrategyId(String strategyId) {
		this.strategyId = strategyId;
	}

	/**
	 * Gets the id of the template used.
	 * 
	 * @return the id of the template used.
	 */
	public String getTemplateId() {
		return templateId;
	}

	/**
	 * Sets the id of the template used
	 * 
	 * @param templateId
	 *            the id of the template used
	 */
	public void setTemplateId(String templateId) {
		this.templateId = templateId;
	}

	/**
	 * Gets the reserve price.
	 * 
	 * @return the reserve price
	 */
	public float getReservePrice() {
		return reservePrice;
	}

	/**
	 * Sets the value of reserve price.
	 * 
	 * @param reservePrice
	 *            the reserve price to set
	 */
	public void setReservePrice(float reservePrice) {
		this.reservePrice = reservePrice;
	}

	/**
	 * Gets the asking price.
	 * 
	 * @return the asking price
	 */
	public float getAskingPrice() {
		return askingPrice;
	}

	/**
	 * Sets the value of asking price.
	 * 
	 * @param askingPrice
	 *            the asking price to set
	 */
	public void setAskingPrice(float askingPrice) {
		this.askingPrice = askingPrice;
	}

	/**
	 * Gets the list of the bids made during the auction.
	 * 
	 * @return the list of the bids made during the auction.
	 */
	public List<BidInfo> getBids() {
		return bids;
	}

	/**
	 * Sets the list of the bids made during the auction.
	 * 
	 * @param bids
	 *            the list of the bids made during the auction
	 */
	public void setBids(List<BidInfo> bids) {
		this.bids = bids;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append(String.format("NegotiationLog: {\n" +
				"\tnegotiationId: %s\n" +
				"\tproductId: %s\n" +
				"\tnegotationStart: %s\n" +
				"\tnegotiationEnd: %s\n" +
				"\tquantityOffered: %s\n" +
				"\tstrategyId: %s\n" +
				"\treservePrice: %s\n" +
				"\ttemplateId: %s\n" +
				"\taskingPrice: %s\n", 
				negotiationId, productId, negotiationStart, negotiationEnd,
				quantityOffered, strategyId, reservePrice,
				templateId, askingPrice));
		if(bids != null)
		{
			sb.append("\tbids: {\n");
			for (BidInfo bi : bids) {
				sb.append("\t\t"+bi);
			}
		}
		sb.append("\t}");
		sb.append("\n}");
		return sb.toString();		
	}
}
