package ibspan.ecap.shared.util;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;

//import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter.DEFAULT;

import jade.util.BasicProperties;
import jade.util.EnhancedProperties;
import jade.util.ExpandedProperties;
import jade.util.Logger;
import jade.util.PropertiesException;

public class PropertiesProvider {
	protected Logger myLogger = Logger.getMyLogger(this.getClass().getName());
	
	private EnhancedProperties properties = null;
	
	private DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
	private static PropertiesProvider instance = new PropertiesProvider();
		
	private PropertiesProvider()
	{}
	
	public static PropertiesProvider getInstance()
	{
		return instance;
	}
	
	public synchronized void loadProperties(String fileName, Object[] args) {
		try {
			if (properties == null) {
				properties = new EnhancedProperties(fileName);
			} else {
				properties.addFromFile(fileName);
			}
			myLogger.info("Loaded properties from file.");

		} catch (FileNotFoundException e) {
			myLogger.log(Level.WARNING, "Couldn't find the properties file: "
					+ fileName, e);

		} catch (IOException ioe) {
			myLogger
					.log(Level.WARNING, "Error occurred while loading properties from a file.", ioe);
			throw new PropertiesException("Failed to load properties file.");
		}
		
// //Copy args to the properties object
//		if (args != null) {
//		    int len = args.length;
//		    String sargs[] = new String[len];
//		    for (int i=0; i < len; i++) {
//		        sargs[i] = args[i].toString();
//		    }
//		    properties.parseArgs(sargs);                
//		}
	}
	
	public synchronized String getPropertyList()
	{
		if (properties != null) {
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw, true);
			properties.list(pw);
			return sw.toString();
		} else
			return "";
	}
	
	public synchronized Date getDateProperty(String key, Date defaultValue)
	{
		if (properties != null) {
			String dateString = properties.getProperty(key);
			if (dateString != null) {
				try {
					return df.parse(dateString);
				} catch (ParseException pe) {
					// fall through
				}
			}
		}
		
		return defaultValue;	//if anything went wrong
	}

	public synchronized double getDoubleProperty(String key,
			double defaultValue) {
		
		if (properties != null) {
			String valueString = properties.getProperty(key);
			if (valueString != null) {
				try {
					return Double.parseDouble(valueString);
				} catch (NumberFormatException nfe) {
					// fall through
				}
			}
		}
		return defaultValue;
	}

	public synchronized String getStringProperty(String key,
			String defaultValue) {
		
		if (properties != null) {
			return properties.getProperty(key, defaultValue);
		} else
			return defaultValue;
	}

	public synchronized long getLongProperty(String key,
			long defaultValue) {
		
		if (properties != null) {
			String valueString = properties.getProperty(key);
			if (valueString != null) {
				try {
					return Long.parseLong(valueString);
				} catch (NumberFormatException nfe) {
					// fall through
				}
			}
		}
		return defaultValue;
	}

	public synchronized int getIntProperty(String key,
			int defaultValue) {
		
		if (properties != null) {
			String valueString = properties.getProperty(key);
			if (valueString != null) {
				try {
					return Integer.parseInt(valueString);
				} catch (NumberFormatException nfe) {
					// fall through
				}
			}
		}
		return defaultValue;
	}
}
